import 'package:example/res/assets_res.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:v_base_framework/page/base_page.dart';
import 'package:v_base_framework/utils/colors_util.dart';
import 'package:v_base_framework/widget/appbar_helper.dart';
import 'package:v_components/widgets/alert/alert.dart';
import 'package:v_components/widgets/alert/alert_config.dart';
import 'package:v_components/widgets/alert/alert_const.dart';
import 'package:v_components/widgets/button/button_widget.dart';
import 'package:v_components/widgets/button/grident_button.dart';

/// FileName showVAlert
///
/// @Author ThinkPad
/// @Date 2023/9/15 8:10
///
/// @Description alert的
///

class VAlertDemo extends BasePage {
  @override
  CommonBasePageState<VAlertDemo> createBaseState() => _ButtonState();
}

class _ButtonState extends CommonBasePageState<VAlertDemo> {
  @override
  void initBaseParams() {
    // TODO: implement initBaseParams
  }

  @override
  void initPageData() {
    // TODO: implement initPageData
  }

  @override
  Widget buildPageWidget(BuildContext context) {
    return Scaffold(
      appBar: CommonAppbarUtil.getAppBar(
        context,
        appBarBackgroundColor: "#0389fa",
        title: "Alert",
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            ElevatedButton(
              child: const Text('最近单用法'),
              onPressed: () => _onBasicAlertPressed(context),
            ),
            ElevatedButton(
              child: const Text('async语法用法'),
              onPressed: () => _onBasicWaitingAlertPressed(context),
            ),
            ElevatedButton(
              child: const Text('自定义动画'),
              onPressed: () => _onCustomAnimationAlertPressed(context),
            ),
            ElevatedButton(
              child: const Text('自定义一个Button'),
              onPressed: () => _onAlertButtonPressed(context),
            ),
            ElevatedButton(
              child: const Text('自定义两个Buttons'),
              onPressed: () => _onAlertButtonsPressed(context),
            ),
            ElevatedButton(
              child: const Text('自定义Alert的Style'),
              onPressed: () => _onAlertWithStylePressed(context),
            ),
            ElevatedButton(
              child: const Text('显示图片'),
              onPressed: () => _onAlertWithCustomImagePressed(context),
            ),
            ElevatedButton(
              child: const Text('Alert内容区域自定义'),
              onPressed: () => _onAlertWithCustomContentPressed(context),
            ),
            ElevatedButton(
              child: const Text('useRootNavigator属性的测试（使用/不使用  Root navigator）'),
              onPressed: () => _onAlertWithRootNavigator(context),
            ),
          ],
        ),
      ),
    );
  }

//===============================================================================================

// 最简单最基础的
  _onBasicAlertPressed(context) {
    VAlert(
      context: context,
      title: "VAlert",
      desc: "这个牛逼的Alert还有其他的牛逼功能，这是最简单的.",
    ).show();
  }

  // alert关闭后才执行后续代码
  _onBasicWaitingAlertPressed(context) async {
    await VAlert(
      context: context,
      title: "VAlert",
      desc: "这个牛逼的Alert的async用法，框关掉才会执行后续语法，避免使用回调",
    ).show();

    // Code will continue after alert is closed.
    debugPrint("Alert closed now.");
  }

//自定义动画
  _onCustomAnimationAlertPressed(context) {
    VAlert(
      context: context,
      title: "VAlert",
      desc: "上面2个是从屏幕底部弹出到中间，这个是自定义动画，直接在屏幕中间渐变",
      alertAnimation: fadeAlertAnimation,
    ).show();
  }

  Widget fadeAlertAnimation(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) {
    return Align(
      child: FadeTransition(
        opacity: animation,
        child: child,
      ),
    );
  }

// 一个按钮
  _onAlertButtonPressed(context) {
    VAlert(
      context: context,
      type: AlertType.error,
      title: "VAlert",
      desc: "自定义点击按钮",
      buttons: [
        VButton(
            onPressed: () => _onCustomAnimationAlertPressed(context),
            color: ColorsUtil.hex2Color(
              "#0389fa",
            ),
            child: const Text(
              "自定义按钮",
              style: TextStyle(color: Colors.white, fontSize: 20),
            ))
      ],
    ).show();
  }

// 多个自定义按钮
  _onAlertButtonsPressed(context) {
    VAlert(
      context: context,
      type: AlertType.warning,
      title: "VAlert",
      desc: "自定义了2个按钮",
      buttons: [
        VButton(
            onPressed: () => Navigator.pop(context),
            color: ColorsUtil.hex2Color(
              "#1AAD19",
            ),
            child: const Text(
              "左按钮",
              style: TextStyle(color: Colors.white, fontSize: 20),
            )),
        VButton(
            onPressed: () => Navigator.pop(context),
            color: ColorsUtil.hex2Color(
              "#F1924E",
            ),
            child: const Text(
              "右按钮",
              style: TextStyle(color: Colors.white, fontSize: 20),
            )),
      ],
    ).show();
  }

//
// // 高级自定义
  _onAlertWithStylePressed(context) {
    // Reusable alert style
    var alertStyle = VAlertConfig(
        animationType: AnimationType.fromTop,
        isCloseButton: false,
        isOverlayTapDismiss: false,
        descStyle: const TextStyle(fontWeight: FontWeight.bold),
        animationDuration: const Duration(milliseconds: 400),
        alertBorder: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(0.0),
          side: const BorderSide(
            color: Colors.grey,
          ),
        ),
        titleStyle: const TextStyle(
          color: Colors.red,
        ),
        constraints: const BoxConstraints.expand(width: 350),
        //First to chars "55" represents transparency of color
        overlayColor: const Color(0x55000000),
        alertElevation: 0,
        alertAlignment: Alignment.topCenter);

    // 高度自定义的
    VAlert(
      context: context,
      style: alertStyle,
      type: AlertType.warning,
      title: "VAlert",
      desc: "自定义Alert的样式：标题红色、背景透明度、弹框宽度、点外边不能关，框加边框、动画",
      buttons: [
        VButton(
            onPressed: () => Navigator.pop(context),
            width: 200,
            radius: BorderRadius.circular(30),
            color: ColorsUtil.hex2Color(
              "#0389fa",
            ),
            child: const Text(
              "自定义按钮",
              style: TextStyle(color: Colors.white, fontSize: 20),
            ))
      ],
    ).show();
  }

//
// // 自定义图片
  _onAlertWithCustomImagePressed(context) {
    VAlert(
      context: context,
      title: "VAlert",
      desc: "图片自定义",
      image: Image.asset(AssetsRes.EMPTY),
    ).show();
  }

//
// // 自定义内容区域
  _onAlertWithCustomContentPressed(context) {
    VAlert(
        context: context,
        title: "LOGIN",
        content: const Column(
          children: <Widget>[
            TextField(
              decoration: InputDecoration(
                icon: Icon(Icons.account_circle),
                labelText: 'Username',
              ),
            ),
            TextField(
              obscureText: true,
              decoration: InputDecoration(
                icon: Icon(Icons.lock),
                labelText: 'Password',
              ),
            ),
          ],
        ),
        buttons: [
          VButton(
              onPressed: () => Navigator.pop(context),
              color: ColorsUtil.hex2Color(
                "#0389fa",
              ),
              radius: const BorderRadius.all(Radius.circular(40)),
              child: const Text(
                "LOGIN",
                style: TextStyle(color: Colors.white, fontSize: 20),
              ))
        ]).show();
  }

//
// // Alert without root navigator
  _onAlertWithRootNavigator(BuildContext context) {
    Navigator.of(context).push(
      PageRouteBuilder(
        pageBuilder: (context, animation, secondaryAnimation) => CupertinoTabScaffold(
          tabBar: CupertinoTabBar(
            items: const <BottomNavigationBarItem>[
              BottomNavigationBarItem(icon: Icon(Icons.account_circle)),
              BottomNavigationBarItem(icon: Icon(Icons.search))
            ],
          ),
          tabBuilder: (BuildContext context, int index) {
            return CupertinoTabView(
              builder: (BuildContext context) {
                return CupertinoPageScaffold(
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      const Text(
                        "新起一个CupertinoTabView, 他有自己的 navigator.（关于这部分讲解请查看笔记1、组件基础/2.1）",
                        style: TextStyle(inherit: false, color: Colors.black),
                        textAlign: TextAlign.center,
                      ),
                      ElevatedButton(
                        child: const Text('VAlert without Root navigator'),
                        onPressed: () => VAlert(
                            context: context,
                            title: "pop() use root navigator ?\n\n* 点取消按钮将仅关闭alert，还停留在这个新起的CupertinoTabView \n* 点确定按钮将关掉alert同时关闭CupertinoTabView",
                            useRootNavigator: false,
                            buttons: [
                              VButton(
                                  width: 200,
                                  onPressed: () => Navigator.of(context).pop(),
                                  color: ColorsUtil.hex2Color(
                                    "#F1924E",
                                  ),
                                  child: const Text(
                                    "取消",
                                    style: TextStyle(color: Colors.white, fontSize: 20),
                                  )),
                              VButton(
                                  width: 200,
                                  onPressed: () => Navigator.of(context, rootNavigator: true).pop(),
                                  color: ColorsUtil.hex2Color(
                                    "#F1924E",
                                  ),
                                  child: const Text(
                                    "确定",
                                    style: TextStyle(color: Colors.white, fontSize: 20),
                                  )),
                            ]).show(),
                      ),
                      ElevatedButton(
                        child: const Text('VAlert with Root navigator'),
                        onPressed: () => VAlert(
                            context: context,
                            title: "pop() use root navigator ?\n\n* 点取消按钮将关闭alert同时会报错 !\n* 点确定将只关闭alert",
                            useRootNavigator: true,
                            buttons: [
                              VButton(
                                onPressed: () => Navigator.of(context).pop(),
                                color: ColorsUtil.hex2Color(
                                  "#F1924E",
                                ) ,
                                  child: const Text(
                                    "取消",
                                    style: TextStyle(color: Colors.white, fontSize: 20),
                                  )
                              ),
                              VButton(
                                onPressed: () => Navigator.of(context, rootNavigator: true).pop(),
                                  color: ColorsUtil.hex2Color(
                                    "#F1924E",
                                  ),
                                  child: const Text(
                                    "确定",
                                    style: TextStyle(color: Colors.white, fontSize: 20),
                                  )),
                            ]).show(),
                      ),
                    ],
                  ),
                );
              },
            );
          },
        ),
      ),
    );
  }
}
