import 'package:flutter/material.dart';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:v_base_framework/http/config/http_config.dart';
import 'package:v_base_framework/http/config/response_config.dart';
import 'package:v_base_framework/http/http.dart';
import 'package:v_base_framework/page/base_page.dart';
import 'package:v_base_framework/widget/appbar_helper.dart';
import 'package:v_components/widgets/cell/cell_widget.dart';
import 'package:v_components/widgets/empty/empty_widget.dart';
import 'package:v_components/widgets/refresh/refresh_widget.dart';
class RefreshDemo extends BasePage {

  // RefreshDemo(
  //    {
  //   Key? key,
  // }) : super(key: key);

  @override
  CommonBasePageState<BasePage> createBaseState() => _RefreshState();
}

class _RefreshState extends CommonBasePageState<RefreshDemo> {
  List _dataArr = [];
  int _pageIndex = 0;
  final int _limit = 15;
  final bool _isGridView = false;//改这个切换listview或者gridview
  GlobalKey<BaseRefreshViewState> globalKeyRefresh = GlobalKey();
  @override
  Widget buildPageWidget(BuildContext context) {
    return Scaffold(
      appBar: CommonAppbarUtil.getAppBar(
        context,
        appBarBackgroundColor: "#0389fa",
        title: _isGridView ? 'GridView分页' : 'ListView分页',
        actionView: InkWell(
          child: const Text('空数据'),
          onTap: () {
            if (!mounted) return;
            setState(() {
              _dataArr = [];
            });
          },
        ),
      ),
      body: _isGridView ? _gridviewBody() : _listviewBody(),
    );
  }

  @override
  void initBaseParams() {
    Http().init(HttpDioConfig(
        successCode: 200,
        tokenInvalidCode: 401,
        errorMessage: "请求失败,请稍后再试!",
        connectTimeout: 10*60*1000,//下载的时候尤其要用
        isNeedProxy: false,
        proxyHost: "172.16.56.219:8615",

        loadingConfig: LoadingConfig(
          loadingViewBackgroundColor: Colors.red,
        )

      // headerInterceptor: TokenInterceptor()
    ));
    ReponseConfigUtils.setResponseConfig(
        ReponseConfig(responseMessageName: "msg"));


    Future.delayed(const Duration(milliseconds: 100), () {
      _requestData(isShowLoading: true);
    });
  }

  @override
  void initPageData() {
  }

  Widget _listviewBody() {
    return VRefreshView(
      key: globalKeyRefresh,
      data: _dataArr,
      // limit: _limit,
      onRefresh: () async => _requestData(),
      onLoad: () async => _requestData(isLoadMore: true),
      itemBuilder: (context, index) => _itemWidget(index),
      separatorBuilder: (context, index) => const Divider(height: 15, indent: 15, endIndent: 15, color: Colors.blue),
    );
  }

  Widget _gridviewBody() {
    Widget child;
    if (_dataArr.isEmpty) {
      child = ListView.builder(
        itemCount: 1,
        itemBuilder: (context, index) => const VEmptyView(),
      );
    } else {
      child = GridView.builder(
        shrinkWrap: true,
        gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
          //可以直接指定每行（列）显示多少个Item
          crossAxisCount: 2, //一行的Widget数量
          crossAxisSpacing: 5, //水平间距
          mainAxisSpacing: 5, //垂直间距
          childAspectRatio: 1.0, //子Widget宽高比例
        ),
        //GridView内边距
        padding: const EdgeInsets.all(5),
        itemCount: _dataArr.length,
        itemBuilder: (context, index) {
          return _itemWidget(index);
        },
      );
    }
    return VRefreshView(
      key: globalKeyRefresh,
      data: _dataArr,
      // limit: _limit,
      onRefresh: () async => _requestData(),
      onLoad: () async => _requestData(isLoadMore: true),
      child: child,
    );
  }


  Widget _itemWidget(index) {
    return InkWell(
      child: Container(
        width: 100,
        height: 100,
        alignment: Alignment.center,
        child: Text(_dataArr[index]['name'],),
      ),
      onTap: () =>{
        debugPrint(_dataArr[index]['name'])
      },
    );
  }


  void _requestData({isShowLoading = false, isLoadMore = false}) {
    _pageIndex = isLoadMore ? _pageIndex + 1 : 0;
    var params = {
      'page': _pageIndex,
      'limit': _limit,
    };
    var loadingText = isShowLoading == true ? 'Loading...' : null;
    globalKeyRefresh.currentState?.request2GetListData(context,"https://www.fastmock.site/mock/1010b262a743f0b06c565c7a31ee9739/root/mock/pages", params:params, loadingText: loadingText, isLoadMore: isLoadMore,
        success: (res) {
          var tempData = res;
          tempData = tempData.isNotEmpty ? tempData : [];
          if (!mounted) return;
          setState(() {
            if (isLoadMore) {
              _dataArr = _dataArr + tempData;
            } else {
              _dataArr = tempData;
            }
          });
        }, fail: (code, msg) {});
  }


}
