import 'alert_const.dart';
import 'package:flutter/material.dart';

/// @author:   wzp_vicky
/// @time:     2023年9月14日08:13:05
/// @version:  V1.1
/// @description: 这里是写给alert的样式
/// @use:AnimationTransition.fromRight(animation, secondaryAnimation, child);
///

class VAlertConfig {
  final AnimationType animationType; //动画：默认是: "fromBottom"
  final Duration animationDuration; //动画持续时长：默认"200 ms"
  final ShapeBorder? alertBorder; //设置边框
  final bool isButtonVisible; //显示或者隐藏按钮
  final bool isCloseButton; //是否显示关闭alert的按钮：默认显示: "true"
  final bool isOverlayTapDismiss;//单击alert外部关闭alert，默认是关闭
  final Color? backgroundColor;
  final Color overlayColor;//遮罩的颜色默认是Color(0xDD000000)
  final TextStyle titleStyle;//标题的样式
  final TextStyle descStyle;//描述的颜色

  final TextAlign titleTextAlign;//标题对齐方式
  final TextAlign descTextAlign;//描述的对齐方式
  final EdgeInsets buttonAreaPadding;//按钮的padding
  final BoxConstraints? constraints;//按钮大小
  final ButtonsDirection buttonsDirection;//按钮组的方向：Row or Col
  final EdgeInsets alertPadding;//alert的padding
  final double? alertElevation;//alert的elevation
  final AlignmentGeometry alertAlignment;//alert dialog的 alignment
  final bool isTitleSelectable;//title text is selectable or not
  final bool isDescSelectable;//desc text is selectable or not.
  final EdgeInsets? titlePadding;//标题的padding
  final EdgeInsets descPadding;//内容的padding






  const VAlertConfig({
    this.animationType = AnimationType.fromBottom,
    this.animationDuration = const Duration(milliseconds: 200),
    this.alertBorder,
    this.isButtonVisible = true,
    this.isCloseButton = true,
    this.isOverlayTapDismiss = true,
    this.backgroundColor,
    this.overlayColor = Colors.black87,
    this.titleStyle = const TextStyle(
        color: Colors.black,
        fontWeight: FontWeight.w500,
        fontStyle: FontStyle.normal),
    this.titleTextAlign = TextAlign.center,
    this.descStyle = const TextStyle(
        color: Colors.black,
        fontWeight: FontWeight.w400,
        fontStyle: FontStyle.normal),
    this.descTextAlign = TextAlign.center,
    this.buttonAreaPadding = const EdgeInsets.all(20.0),
    this.constraints,
    this.buttonsDirection = ButtonsDirection.row,
    this.alertElevation,
    this.alertPadding = defaultAlertPadding,
    this.alertAlignment = Alignment.center,
    this.isTitleSelectable = false,
    this.isDescSelectable = false,
    this.titlePadding,
    this.descPadding = const EdgeInsets.all(0.0),
  });
}