///  有多种按钮：https://www.jianshu.com/p/31424d1de955
///  这个组件呢，主要针对textbutton的封装
///  TextButton即文本按钮，默认背景透明并不带阴影。按下后，会有背景色
///
/// MaterialButton: 默认按钮，扁平，背景透明，按下后，会有背景色。
// RaisedButton:  已经弃用
// FlatButton： 已经弃用
// IconButton:  图标按钮，只能是纯图标，按钮不可展示文案。
// FloatingActionButton:  浮动按钮，可显示文字和图标。
// OutlineButton:  外边框按钮，可设置按钮外边框颜色。
// ButtonBar:  水平布局的按钮容器，可放置多个Button或Text。
// TextButton：  文字按钮，按钮中带文字
// TextButton.icon():  带图标文字混合按钮，它可以轻松创建带图标和文字的按钮。
//

// 著作权归作者所有。商业转载请联系作者获得授权，非商业转载请注明出处。
///
/// @author:   wzp_vicky
/// @time:     2022年10月27日00:01:09
/// @version:  V1.1
/// @description: 目前支持图片按钮(去掉了水波纹)和纯色按钮

import 'package:flutter/material.dart';

import '../../utils/colors_util.dart';

const double _borderWidth = 2.0; // 边框宽度
const double _borderRadius = 5.0; // 边框圆角

class VTextButton extends StatelessWidget {
  final double height;
  final double width;
  final double fontSize;
  final String? textColor;
  final String text;
  final String? disabledBgColor;
  final String? bgColor;
  final String? imageBg; //图片背景的按钮
  final VoidCallback? onPressed; //不传这个就会禁用
  final bool hasBorder;
  final TextStyle? textStyle;
  final BorderRadius? radius;
  final Gradient? gradient;

  const VTextButton({
    Key? key,
    this.height = 51,
    this.text = '',
    this.width = double.infinity,
    this.textColor,
    this.fontSize = 16,
    this.disabledBgColor,
    this.bgColor = "#cd0000",
    this.imageBg,
    this.onPressed, // 传空为disabled状态
    this.hasBorder = false,
    this.textStyle,
    this.radius = const BorderRadius.all(Radius.circular(6)),
    this.gradient
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    var _textColor = ColorsUtil.hex2Color(textColor ?? "#ffffff");
    var disabledTextColor = _textColor.withOpacity(0.6); // disabled按钮文字颜色
//https://cloud.tencent.com/developer/article/1853297  改阴影的时候使用
    return imageBg != null
            ? Container(
                height: height,
                width: width,
                decoration: BoxDecoration(
                  image: DecorationImage(image: AssetImage(imageBg!)),
                ),
                child: MaterialButton(
                  onPressed: onPressed,
                  minWidth: 60,
                  // highlightColor: Colors.transparent,
                  child: Text(text, style: textStyle ?? TextStyle(fontSize: fontSize, color: _textColor)),
                ),
              )
            : Container(
                width: width,
                height: height,
                decoration: BoxDecoration(
                  borderRadius: radius,

                ),
                child: TextButton(
                  onPressed: onPressed,
                  style: ButtonStyle(
                    // 设置按钮大小
                    minimumSize: MaterialStateProperty.all(Size(double.infinity, height)),

                    // 背景色
                    backgroundColor: MaterialStateProperty.resolveWith(
                      (states) {
                        if (states.contains(MaterialState.disabled)) {
                          // disabled状态颜色
                          return ColorsUtil.hex2Color(disabledBgColor ?? "#cd0000");
                        } else if (states.contains(MaterialState.pressed)) {
                          //按下时的颜色
                          // return Colors.redAccent;
                        }
                        // 默认状态颜色
                        return ColorsUtil.hex2Color(bgColor!);
                      },
                    ),
                    overlayColor: MaterialStateProperty.all(ColorsUtil.hex2Color(bgColor!, alpha: 0.7)),
                    // 文字颜色
                    foregroundColor: MaterialStateProperty.resolveWith(
                      (states) {
                        if (states.contains(MaterialState.disabled)) {
                          // disabled状态颜色
                          return disabledTextColor;
                        } else if (states.contains(MaterialState.pressed)) {
                          //按下时的颜色
                          // return Colors.yellow;
                        }
                        // 默认状态颜色
                        return _textColor;
                      },
                    ),
                    // 边框
                    side: !hasBorder
                        ? null
                        : MaterialStateProperty.all(
                            BorderSide(color: ColorsUtil.hex2Color(disabledBgColor ?? "#0389fa"), width: hasBorder ? _borderWidth : 0),
                          ),
                    // 圆角
                    shape: MaterialStateProperty.all(
                      RoundedRectangleBorder(borderRadius: radius!),
                    ),
                  ),
                  child: Text(text, style: TextStyle(fontSize: fontSize)),
                ),
              )

        /*TextButton(
            onPressed: onPressed,
            style: ButtonStyle(
              // 设置按钮大小
              minimumSize: MaterialStateProperty.all(Size(double.infinity, height)),

              // 背景色
              backgroundColor: MaterialStateProperty.resolveWith(
                (states) {
                  if (states.contains(MaterialState.disabled)) {
                    // disabled状态颜色
                    return ColorsUtil.hex2Color(disabledBgColor ?? "#cd0000");
                  } else if (states.contains(MaterialState.pressed)) {
                    //按下时的颜色
                    // return Colors.redAccent;
                  }
                  // 默认状态颜色
                  return ColorsUtil.hex2Color(bgColor!);
                },
              ),
              overlayColor: MaterialStateProperty.all(ColorsUtil.hex2Color(bgColor!, alpha: 0.7)),
              // 文字颜色
              foregroundColor: MaterialStateProperty.resolveWith(
                (states) {
                  if (states.contains(MaterialState.disabled)) {
                    // disabled状态颜色
                    return disabledTextColor;
                  } else if (states.contains(MaterialState.pressed)) {
                    //按下时的颜色
                    // return Colors.yellow;
                  }
                  // 默认状态颜色
                  return _textColor;
                },
              ),
              // 边框
              side: !hasBorder
                  ? null
                  : MaterialStateProperty.all(
                      BorderSide(color: ColorsUtil.hex2Color(disabledBgColor ?? "#0389fa"), width: hasBorder ? _borderWidth : 0),
                    ),
              // 圆角
              shape: MaterialStateProperty.all(
                RoundedRectangleBorder(borderRadius: BorderRadius.circular(_borderRadius)),
              ),
            ),
            child: Text(text, style: TextStyle(fontSize: fontSize)),
          )*/
        ;
  }
}
