import 'package:flutter/material.dart';

import '../../utils/colors_util.dart';
import 'field_widget.dart';

/// @author:   wzp_vicky
/// @time:     2022年10月25日17:38:28
/// @version:  V1.1
/// @description:仿有赞组件的cell
/// */
///
const double _lineHeight = 0.6; // 底部线高

class VCell extends StatefulWidget {
  final String title; //左侧标题
  final String value; //右侧内容
  final String? hintText; //提示文字（eg：请选择（城市）-->西安市，默认不要）
  final String? labelText; // top提示文字
  final String? errorText; // 错误提示文字
  final bool showRedStar; // 显示左侧小红星，默认不显示
  final bool hiddenArrow; // 隐藏箭头，默认不隐藏
  final Widget? leftWidget; // 左侧widget ，默认隐藏
  final Widget? rightWidget; // 右侧widget ，默认隐藏
  final VoidCallback? clickCallBack;
  final double space; // 标题宽度
  final TextStyle? titleStyle;
  final TextStyle? textStyle;
  final TextStyle? hintTextStyle;
  final TextStyle? labelTextStyle; // 默认为hintTextStyle，高亮为主题色
  final TextAlign textAlign; // 输入文字对齐方式，默认左对齐
  final InputBorder border; // 输入边框样式，默认无边框
  final bool hiddenLine; // 隐藏底部横线
  final bool topAlign; // 左侧标题顶部对齐，默认居中
  final String? bgColor; // 背景颜色，默认白色  只需要传色值：eg:#cd0000
  final String lineColor; // 背景颜色，默认白色  只需要传色值：eg:#cd0000
  final double cellHeight; // 输入、选择样式一行的高度
  final double radius; //cell的圆角  默认无值
  final bool isTransparentBg;//透明背景，为了适应使用的时候圆角问题

  @override
  _VCellState createState() => _VCellState();

  const VCell(
      {Key? key,
      this.title = '',
      this.value = '',
      this.hintText,
      this.labelText,
      this.errorText,
      this.leftWidget,
      this.rightWidget,
      this.showRedStar = false,
      this.hiddenArrow = false,
      this.border = InputBorder.none, // 去掉下划线
      this.clickCallBack,
      this.space = 100,
      this.hiddenLine = false,
      this.topAlign = false,
      this.bgColor,
      this.lineColor = "#dcdcdc",
      this.cellHeight = 50,
      this.textAlign = TextAlign.left,
      this.titleStyle,
      this.textStyle,
      this.hintTextStyle,
      this.labelTextStyle,
        this.isTransparentBg=false,
      this.radius = 0})
      : super(key: key);
}

class _VCellState extends State<VCell> {
  bool _hiddenArrow = false;

  // double _topSpace = 0; // title 顶对齐 间距

  @override
  void initState() {
    super.initState();

    _hiddenArrow = widget.hiddenArrow;
    if (widget.border != InputBorder.none) {
      _hiddenArrow = true;
    }
  }

  @override
  Widget build(BuildContext context) {
    double _starW = !widget.showRedStar && widget.title.isEmpty ? 0 : 8;
    // double _starW = 0;
    double _topSpace = 0; // title 顶对齐 间距
    return Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.all(Radius.circular(widget.radius)),
          color:widget.isTransparentBg?Colors.transparent: ColorsUtil.hex2Color(widget.bgColor ?? "#ffffff"),
        ),
        child: InkWell(
          child: Container(
              constraints: BoxConstraints(
                  minWidth: double.infinity, // 宽度尽可能大
                  minHeight: widget.cellHeight // 最小高度为50像素
                  ),
              padding: const EdgeInsets.fromLTRB(5, 0, 10, 0),
              decoration: UnderlineTabIndicator(
                borderSide: BorderSide(
                    width: _lineHeight,
                    color: widget.hiddenLine
                        ? Colors.transparent
                        : ColorsUtil.hex2Color(widget.lineColor)),
                // insets: EdgeInsets.fromLTRB(_starW, 0, 0, 0)
              ),
              child: Row(
                  crossAxisAlignment: widget.topAlign
                      ? CrossAxisAlignment.start
                      : CrossAxisAlignment.center,
                  children: <Widget>[
                    widget.leftWidget ?? Container(),
                    Container(
                      width: _starW,
                      padding: EdgeInsets.fromLTRB(
                          0, widget.topAlign ? _topSpace : 0, 0, 0),
                      child: Text(widget.showRedStar ? '*' : ' ',
                          style: const TextStyle(
                              fontSize: 18.0, color: Colors.red)),
                    ),
                    Offstage(
                      offstage: widget.title.isEmpty,
                      child: Container(
                        width: widget.space - _starW,
                        padding: EdgeInsets.fromLTRB(
                            0, widget.topAlign == true ? _topSpace : 0, 0, 0),
                        child: Text(widget.title, style: widget.titleStyle),
                      ),
                    ),
                    Expanded(
                        //为了达到有hint的效果，所以使用这个
                        child: VField(
                      text: widget.value,
                      hintText: widget.hintText ?? '',
                      // labelText: widget.labelText,
                      errorText: widget.errorText,
                      enabled: false,
                      textStyle: widget.textStyle,
                      hintTextStyle: widget.hintTextStyle,
                      // labelTextStyle: widget.labelTextStyle,
                      textAlign: widget.textAlign,

                      border: widget.border,
                    )),
                    widget.rightWidget ?? Container(),
                    Offstage(
                      offstage: _hiddenArrow,
                      child: const Icon(Icons.arrow_forward_ios,
                          size: 18, color: Color(0xFFC8C8C8)),
                    ),
                  ])),
          onTap: () {
            if (widget.clickCallBack != null) {
              widget.clickCallBack!();
            }
          },
        ));
  }
}
