import 'package:flutter/material.dart';

import 'drop_menu_controller.dart';


/// 下拉组件头部 配合DropDownMenu 使用
class DropDownMenuHeader extends StatefulWidget {
  /// 菜单控制器
  final DropDownMenuController menuController;

  /// 头部显示数组
  final List<String> titles;

  /// 头部高度
  final double? headHeight;

  /// 默认文字 icon 色值
  final Color defaultColor;

  /// 点击色值
  final Color clickColor;

  /// icon大小
  final double? iconSize;

  /// 右边Icon
  final AssetImage? drawableEndAssetImg;

  const DropDownMenuHeader({
    Key? key,
    required this.menuController,
    required this.titles,
    this.headHeight,
    this.defaultColor = const Color(0xFF333333),
    this.clickColor = const Color(0xFF333333),
    // this.clickColor = const Color(0xFFFF00FF),
    this.drawableEndAssetImg,
    this.iconSize,
  }) : super(key: key);

  @override
  _DropDownMenuHeaderState createState() => _DropDownMenuHeaderState();
}

class _DropDownMenuHeaderState extends State<DropDownMenuHeader> {
  final _children = <Widget>[];

  var currentIndex = -1;

  //首先我们的头部在没有选择选项的时候会有一个默认的数组，这个是永远不会改变的，所以这个数组一旦设置了就不能改变了，之后我们新建一个动态数组也就是当前显示的数组，这个数组的默认值就是我们未选择选项的默认值，这里我们需要监听头部状态的改变来对显示数组进行处理

  // 筛选头部显示的文本数组
  List<String> changeTitles = [];

  @override
  void initState() {
    super.initState();
    changeTitles.addAll(widget.titles);
    for (var i = 0; i < changeTitles.length; i++) {
      _children.add(searchFilter(changeTitles[i], i)); //头部组件数组
    }
    widget.menuController.addListener(() {
      // 下拉 true 隐藏 false
      var isShow = widget.menuController.isShow;
      setState(() {
        if (widget.menuController.title != "") {
          // 说明当前选择了选项 赋值我选择的选项
          changeTitles[widget.menuController.titleIndex] =
              widget.menuController.title;
        } else {
          // 为空 说明当前的选项我清空了 赋值初始头部数组的数据
          changeTitles[widget.menuController.titleIndex] =
              widget.titles[widget.menuController.titleIndex];
        }

        // currentIndex 当前选择的index 默认-1 用来对比更新头部文字和颜色
        // 如果下拉 更新当前选项inedx 如果隐藏说明没有选择任何一个下拉框 置为-1
        if (isShow && currentIndex < widget.titles.length) {
          currentIndex = widget.menuController.index;
        } else {
          currentIndex = -1;
        }

        // 每次下拉/收回我们只需改变头部数据即可 changeTitles 永远都是显示的数组 直接全部更新到组件即可
        _children.clear();
        for (var i = 0; i < changeTitles.length; i++) {
          _children.add(searchFilter(changeTitles[i], i));
        }
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(

      height: widget.headHeight ?? 45,
      child: Row(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.start,
          children: _children),
    );
  }

  Widget searchFilter(String name, int index) {
    return InkWell(
      child: Container(
        color: Colors.transparent,
        margin: const EdgeInsetsDirectional.only(start: 10),
        alignment: Alignment.center,
        child: RichText(
            maxLines: 1,
            overflow: TextOverflow.ellipsis,
            text: TextSpan(
                text: name,
                style: TextStyle(
                    color: currentIndex == index || widget.titles[index] != name
                        ? widget.clickColor
                        : widget.defaultColor),
                children: [
                  WidgetSpan(
                      child: widget.drawableEndAssetImg == null
                          ? Icon(
                              currentIndex == index
                                  ? Icons.arrow_drop_up
                                  : Icons.arrow_drop_down,
                              color: currentIndex == index ||
                                      widget.titles[index] != name
                                  ? widget.clickColor
                                  : widget.defaultColor,
                              size: widget.iconSize ?? 20,
                            )
                          : ImageIcon(
                              widget.drawableEndAssetImg,
                              color: currentIndex == index ||
                                      widget.titles[index] != name
                                  ? widget.clickColor
                                  : widget.defaultColor,
                              size: widget.iconSize ?? 20,
                            ))
                ])),
      ),
      onTap: () {
        if (index == currentIndex && widget.menuController.isShow) {
          widget.menuController.hide();
        } else {
          widget.menuController.show(index);
          print("index $index");
        }
      },
    );
  }
}
