import 'package:flutter/material.dart';

import '../../res/assets_res.dart';
import '../../utils/colors_util.dart';

///
/// @author:   wzp_vicky
/// @time:     2022年11月2日22:51:15
/// @version:  V1.1
/// @description: 空数据组件。暂无数据 和 网络加载失败两种
///
const String _emptyText = '暂无数据';
const String _networkErrorText =
    "请求失败，点击重新加载"; //'网络不给力，点击重新加载'; 因为目前不牵涉网络的判断，所以这的文字默认成“请求失拜，点击重新加载”
const String _btnTitle = '点击重试'; // 重新加载

const String _emptyImagePath = AssetsRes.EMPTY;
const String _networkErrorImagePath = AssetsRes.EMPTY;

const double _space = 20.0; // 控件垂直间距
const double _btnBorderRadius = 20.0; // 边框圆角
const double _textFontSize = 16.0; // 文字大小
const double _btnFontSize = 17.0; // 按钮文字大小

enum EmptyType { empty, error, loading }

class VEmptyView extends StatefulWidget {
  final EmptyType type; // 空数据类型
  final String? image; // 空数据图片,不传根据类型显示默认图片
  final double space; // 图片和文字垂直间距
  final VoidCallback? clickCallBack; // 按钮点击回调
  final String? text; // 描述文字,不传根据类型显示默认文字
  final String reloadBtnTxt; // 当类型为error时，显示按钮
  final String? reloadBtnBgColor;
  final double reloadBtnTxtFontSize;
  final double infoFontSize;
  final String? infoTextColor;

  const VEmptyView({
    Key? key,
    this.type = EmptyType.empty,
    this.image,
    this.text,
    this.reloadBtnTxt = _btnTitle,
    this.space = _space,
    this.clickCallBack,
    this.reloadBtnBgColor,
    this.reloadBtnTxtFontSize = _btnFontSize,
    this.infoFontSize = _textFontSize,
    this.infoTextColor,
  }) : super(key: key);

  @override
  State<VEmptyView> createState() => _VEmptyViewState();
}

class _VEmptyViewState extends State<VEmptyView> {
// 默认颜色
  var textColor = Color(0xFFB1BBC3);

  @override
  Widget build(BuildContext context) {
    var image = widget.image ??
        (widget.type == EmptyType.error
            ? _networkErrorImagePath
            : _emptyImagePath);
    var text = widget.text ??
        (widget.type == EmptyType.error ? _networkErrorText : _emptyText);

    return  Center(child: Column(

      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        image.isNotEmpty ? Image.asset(image,width: 200,height: 150,) : Container(),
        SizedBox(height: image.isNotEmpty ? widget.space : 0),
        Text(text,
            style: TextStyle(
                fontSize: widget.infoFontSize,
                color: widget.infoTextColor != null
                    ? ColorsUtil.hex2Color(widget.infoTextColor!)
                    : textColor)),
        SizedBox(height: widget.type == EmptyType.error ? widget.space : 0),
        Visibility(
          visible: widget.type == EmptyType.error,
          child: InkWell(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 50),
              width: double.infinity,
              height: 40,
              decoration: BoxDecoration(
                  color: ColorsUtil.hex2Color(
                      widget.reloadBtnBgColor ?? "#0389fa"),
                  borderRadius: BorderRadius.circular(_btnBorderRadius)),
              child: Center(
                  child: Text(widget.reloadBtnTxt,
                      style: TextStyle(
                          fontSize: widget.reloadBtnTxtFontSize,
                          color: Colors.white))),
            ),
            onTap: () => widget.clickCallBack?.call(),
          ),
        ),
      ],
    )
      );
  }
}
