/// @author:   wzp_vicky
/// @time:     2022年10月25日22:29:04
/// @version:  V1.1
/// @description:单行输入框
///
/// 当与cell配合使用左边的宽度计算//100（左边的宽度）-5（*的内距）-8(*的宽度)-6 （前面的）
/// */
///
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import '../../utils/colors_util.dart';

const double _lineHeight = 1; // 底部线高
const double _textFontSize = 15.0;

/// 录入回调
typedef _InputCallBack = void Function(String value);

/// 录入完成回调（失去焦点或者点击键盘右下角按钮触发）
/// isSubmitted：是否通过onSubmitted方法触发
/// 直接使用回调范围更大，可判断外部三方键盘关闭按钮点击事件，如果有多个textField切换，也会走这个回调，按需使用
typedef _InputCompletionCallBack = void Function(
    String value, bool isSubmitted);

class VField2 extends StatefulWidget {
  const VField2(
      {Key? key,
        this.text = '',
        this.hintText = '',
        this.labelText,
        this.controller, //有text呢，还有回调事件  这个可以在页面不传
        this.keyboardType = TextInputType.text,
        this.textInputAction = TextInputAction.done,
        this.focusNode,
        this.isPwd = false,
        this.leftWidget,
        this.rightWidget,
        this.maxLength = 20,
        this.isShowDeleteBtn = false,
        this.inputFormatters,
        this.inputCallBack,
        this.inputCompletionCallBack,
        this.pwdOpen,
        this.pwdClose,
        this.border,
        this.isDense = false, //设置为true则输入框的文本垂直方向空隙更小
        this.contentPadding,
        this.showRedStar = false,
        this.hintFontSize = 17,
        this.hintColor,
        this.pwdEyeSize = 22,
        this.pwdEyeColor,
        this.isShowBorder = true,
        this.focusBorderColor,
        this.cursorColor,
        this.unfocusBorderColor})
      : super(key: key);

  final String? text;

  final String? labelText; // top提示文字
  final TextEditingController? controller;
  final TextInputType keyboardType;
  final TextInputAction? textInputAction; // 键盘右下角按钮类型
  final FocusNode? focusNode;
  final bool isPwd; // 是否是密码，默认不是
  final Widget? leftWidget; // 左侧widget ，默认隐藏
  final Widget? rightWidget; // 右侧widget ，默认隐藏
  final int maxLength; // 最大长度，默认20
  final bool isShowDeleteBtn; // 是否显示右侧删除按钮，默认不显示
  final List<TextInputFormatter>? inputFormatters;
  final _InputCallBack? inputCallBack;
  final _InputCompletionCallBack? inputCompletionCallBack;

  final InputBorder? border; // 边框样式
  final bool isDense; // 是否紧凑显示，默认false
  final EdgeInsetsGeometry?
  contentPadding; //内边距    当父组件固定高度时，文本一行显示文本过多会出现文字显示不全bug,可设置EdgeInsets.symmetric(vertical: 4)
  final String? focusBorderColor; //边线聚焦时的颜色
  final String? unfocusBorderColor; //边线不聚焦时的颜色
  final bool showRedStar;

  final String hintText;
  final double hintFontSize;
  final String? hintColor;
  final String? cursorColor;

  final double pwdEyeSize; //眼睛的大小
  final String? pwdEyeColor; //眼睛的颜色
  final String? pwdOpen; // 自定义密码图片路径 睁眼
  final String? pwdClose; // 自定义密码图片路径 闭眼
  final bool isShowBorder;

  @override
  _VField2State createState() => _VField2State();
}

class _VField2State extends State<VField2> {
  TextEditingController? _textController;
  FocusNode? _focusNode;
  bool _isFocused = false;
  bool? _isShowDelete;
  bool? _isHiddenPwdBtn; // 是否隐藏 右侧密码明文切换按钮 ，密码样式才显示（isPwd =true），
  bool? _pwdShow; // 控制密码 明文切换
  Widget? _pwdImg; // 自定义密码图片
  bool _isSubmitted = false; // 记录是否点击键盘右下角按钮

  @override
  void initState() {
    super.initState();

    _textController = widget.controller ?? TextEditingController();
    _textController!.text = widget.text ?? '';
    // 超过最大长度截取
    if ((widget.text ?? '').length > widget.maxLength) {
      _textController!.text =
          (widget.text ?? '').substring(0, widget.maxLength);
    }
    _focusNode = widget.focusNode ?? FocusNode(); //焦点控制
    _isHiddenPwdBtn = !widget.isPwd;
    _pwdShow = widget.isPwd;

    _isShowDelete = _focusNode!.hasFocus && _textController!.text.isNotEmpty;
    _textController!.addListener(() {
      setState(() {
        _isShowDelete =
            _textController!.text.isNotEmpty && _focusNode!.hasFocus;
      });
    });
    _focusNode!.addListener(() {
      setState(() {
        _isFocused = _focusNode!.hasFocus;
        _isShowDelete =
            _textController!.text.isNotEmpty && _focusNode!.hasFocus;
        // 录入完成回调，失去焦点并且不是点击键盘右下角时触发
        if (!_isFocused && !_isSubmitted) {
          widget.inputCompletionCallBack?.call(_textController!.text, false);
        }
      });
    });
  }

  @override
  void didUpdateWidget(covariant VField2 oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);

    var cursorPos = _textController!.selection;
    // 更新text值到_textController
    _textController!.text = widget.text ?? '';
    // 超过最大长度截取
    if ((widget.text ?? '').length > widget.maxLength) {
      _textController!.text =
          (widget.text ?? '').substring(0, widget.maxLength);
    }
    if (cursorPos.start > _textController!.text.length) {
      // 光标保持在文本最后
      cursorPos = TextSelection.fromPosition(
          TextPosition(offset: _textController!.text.length));
    }
    _textController!.selection = cursorPos;
  }

  @override
  void dispose() {
    _focusNode!.unfocus();
    _textController!.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    double _starW = !widget.showRedStar ? 0 : 8;
    if (widget.pwdOpen != null && widget.pwdClose != null) {
      if (widget.pwdOpen!.isNotEmpty && widget.pwdClose!.isNotEmpty) {
        _pwdImg = _pwdShow!
            ? ImageIcon(AssetImage(widget.pwdClose!))
            : ImageIcon(AssetImage(widget.pwdOpen!));
      } else {
        _pwdImg = Icon(_pwdShow! ? Icons.visibility_off : Icons.visibility);
      }
    } else {
      _pwdImg = Icon(_pwdShow! ? Icons.visibility_off : Icons.visibility);
    }

    var textStyle = TextStyle(
        fontSize: _textFontSize, color: ColorsUtil.hex2Color("#333333"));
    var hintTextStyle = TextStyle(
        fontSize: widget.hintFontSize,
        color: ColorsUtil.hex2Color(widget.hintColor ?? "#999999"));

    return Container(
      decoration: widget.isShowBorder
          ? UnderlineTabIndicator(
          borderSide: BorderSide(
              width: _lineHeight,
              color: _isFocused
                  ? ColorsUtil.hex2Color(
                  widget.focusBorderColor ?? "#cd0000")
                  : ColorsUtil.hex2Color(
                  widget.unfocusBorderColor ?? "#f2f2f2")))
          : null,
      child: Row(
        children: [
          SizedBox(
            width: _starW,
            child: Padding(
              padding: const EdgeInsets.only(
                left: 5,
              ),
              child: Text(widget.showRedStar ? '*' : ' ',
                  style: const TextStyle(fontSize: 18.0, color: Colors.red)),
            ),
          ),
          widget.leftWidget == null
              ? Container()
              : Padding(
            padding: EdgeInsets.only(left: 6),
            child: widget.leftWidget,
          ),
          // widget.leftWidget ?? Container(), //图标，防止获焦后变色
          Offstage(
            offstage: widget.leftWidget == null,
            child: const SizedBox(
              width: 0,
            ),
          ),
          Expanded(
            child: Container(
              // color: Colors.yellow,
              child: TextField(
                focusNode: _focusNode,
                controller: _textController,
                keyboardType: widget.keyboardType,
                textInputAction: widget.textInputAction,
                style: textStyle,
                inputFormatters: widget.inputFormatters ??
                    [LengthLimitingTextInputFormatter(widget.maxLength)],
                //https://www.jianshu.com/p/5485e14dc77b
                cursorColor:
                ColorsUtil.hex2Color(widget.cursorColor ?? "#cd0000"),
                decoration: InputDecoration(
                  // isCollapsed:true,
                  //内间距
                  contentPadding: widget.contentPadding ?? EdgeInsets.all(10),
                  //通过这个设置高度，暂时不对外提供属性
                  prefixIcon: null,
                  //头部图标，位于输入框内部最左侧
                  labelText: widget.labelText,
                  //提示语，位于输入框上方
                  hintText: widget.hintText,
                  hintStyle: hintTextStyle,
                  // labelStyle: _isFocused ? labelTextStyle : hintTextStyle,
                  //输入框是否为密集形式，默认为false
                  isDense: widget.isDense,

                  enabledBorder: InputBorder.none,
                  focusedBorder: InputBorder.none,

                  //输入框可用时的边框
                  // enabledBorder: widget.border ??
                  //     UnderlineInputBorder(
                  //         borderSide:
                  //         BorderSide(color: ColorsUtil.hex2Color(
                  //             "#dcdcdc"), width: _lineHeight)),
                  // 输入框开始输入时的边框
                  // focusedBorder: widget.border ??
                  //     UnderlineInputBorder(
                  //         borderSide:
                  //         BorderSide(color: underlineColor, width: _lineHeight)),
                  // suffixIcon: Container(), //如果通过suffixIcon添加右侧自定义widget点击会弹出键盘
                ),
                obscureText: _pwdShow!,
                // 执行顺序为 onTap -> onChanged -> onEditingComplete -> onSubmitted
                // 点击输入框
                onTap: () {
                  _isSubmitted = false;
                },
                // 每次输入框文字改变，均会执行
                onChanged: (value) {
                  widget.inputCallBack?.call(_textController!.text);
                },
                // 输入完成，提交按钮点击后会先执行这里
                onEditingComplete: () {
                  _focusNode!.unfocus();
                },
                // 提交按钮点击
                onSubmitted: (value) {
                  _isSubmitted = true;
                  widget.inputCompletionCallBack
                      ?.call(_textController!.text, true);
                },
              ),
            ),
          ),
          Row(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Offstage(
                offstage: !widget.isShowDeleteBtn,
                child: _isShowDelete!
                    ? Container(
                  // color: Colors.redAccent,
                    width: 30, //20+83333333333
                    child: IconButton(
                        alignment: Alignment.centerLeft,
                        // padding: const EdgeInsets.only(right: 10),
                        padding: const EdgeInsets.all(0),
                        icon: const Icon(
                          Icons.cancel,
                          color: Color(0xFFC8C8C8),
                          size: 20,
                        ),
                        onPressed: () {
                          _textController!.text = '';
                          if (widget.inputCallBack != null) {
                            widget.inputCallBack!(_textController!.text);
                          }
                        }))
                    : const Text(''),
              ),
              Offstage(
                offstage: _isHiddenPwdBtn!,
                child: Container(
                  // color: Colors.blue,
                  // padding: EdgeInsets.all(0),
                  width: widget.pwdEyeSize + 10,
                  child: IconButton(
                    //内边距调小无效果 、、https://blog.csdn.net/weixin_43298955/article/details/115720294
                    //所以采用size的值就是父元素的宽度就行了，剩下的间距自己开心就好
                    alignment: Alignment.centerLeft,
                    icon: _pwdImg!,
                    iconSize: widget.pwdEyeSize,
                    padding: const EdgeInsets.all(0),
                    color:
                    ColorsUtil.hex2Color(widget.pwdEyeColor ?? "#999999"),
                    onPressed: () {
                      setState(() {
                        _pwdShow = !_pwdShow!;
                      });
                    },
                  ),
                ),
              ),
              widget.rightWidget ?? Container(),
            ],
          ),
        ],
      ),
    );
  }
}
