import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import '../../utils/colors_util.dart';
import '../daydart/daydart.dart';
import '../dropDown/drop_down_menu.dart';
import '../dropDown/drop_down_menu_header.dart';
import '../dropDown/drop_menu_controller.dart';
import '../picker/picker_tool.dart';
import '../search/VSearch.dart';

/// @author:   wzp_vicky
/// @time:    时间
/// @version:  V1.1
/// @description:时间筛选组件:主体是时间选择(时间选项可配),配有搜索框，同行显示
///   这个组件的功能跟给小程序封装的timeFilter组件的功能完全相同，只是没有小程序的场景多、功能完善，但是满足基本场景（e6）
///
const List _dayTypeList = [
  {"text": '今天', "value": 1},
  {"text": '昨天', "value": 2},
  {"text": '3天内', "value": 3},
  {"text": '7天内', "value": 7},
  {"text": '自定义', "value": 10086},
];
double _customElementHeight = 25;
double _pickerItemHeight = 40;
Map _res = {
  "beginTime": "",
  "endTime": "",
  "filterKeywords": "",
  "isAttached": true,
  //最早为了啥忘了 这里暂时定义为1.组件绑定进页面的时候2、点的非自定义3、点自定义的搜索的时候为true,点自定义的和选时间的时候为false这是为了选自定义的时候不发请求
  "isCustomDate": false
  //isCustomDate: false  是否是自定义，为了更改scrollview和empty的高度
};

class TimeFormat {
  static Map formats = {
    "format1": "yyyy-MM-dd HH:mm:ss",
    "format2": "yyyy-MM-dd HH:mm",
    "format3": "yyyy-MM-dd",
  };
}

class VTimeFilter extends StatefulWidget {
  final List daySelect;
  int selectDayIndex; //默认选择第几个  eg:默认7天内   改成必传  因为牵涉刷新后组件的重建，所以的吧之前选的传进来
  String? customBgColor;
  String? cursorColor;
  String? filterBgColor; //filter整体的背景色，必须的设置
  String searchHint;
  String selectColor;
  int searchLength=10;

  final List<Widget>? children; //主体内容组件

  late String _pickerTitle;

  Function? timeFilterSelect;
  // String _filterKeywords = "";
  String? format;
  late PickerDateType _pickerType;
  @required Function checkCallBack;

  VTimeFilter(
      this.checkCallBack,
      {this.daySelect = _dayTypeList,
        this.selectDayIndex =
        3, //小程序这里传几对应的是选项的value   因为有赞组件有处理，所以flutter需要处理,在这，这个值专业指的是数组的index 默认是7天内
        this.customBgColor = "#f3f4f5",
        this.filterBgColor = "#f3f4f5",
        this.children,
        String? format,
        this.timeFilterSelect,
        this.cursorColor = "#039aaf2",
        this.searchHint = "请输入",
        this.searchLength=10,
        this.selectColor = "#6c69ff"}) {
    this.format = format ?? TimeFormat.formats["format1"];

    if (this.format == TimeFormat.formats["format1"]) {
      _pickerType = PickerDateType.YMD_HMs;
    } else if (this.format == TimeFormat.formats["format2"]) {
      _pickerType = PickerDateType.YMD_HM;
    } else {
      _pickerType = PickerDateType.YMD;
    }

    _pickerTitle = daySelect[selectDayIndex]["text"];
    // _endTime = DayDart().format(this.format);
    // int gap=daySelect[selectDayIndex]["value"];
    // int gapDP = gap - 1;
    // _beginTime = DayDart().subtract(gapDP, DayUnits.D).format(this.format);

    // if (timeFilterSelect != null) {
    //   _res["beginTime"] = _beginTime;
    //   _res["endTime"] = _endTime;
    //   _res["filterKeywords"] = _filterKeywords;
    //   _res["endTimeMill"] = DayDart(_endTime).toDate().millisecondsSinceEpoch;
    //   _res["beginTimeMill"] =
    //       DayDart(_beginTime).toDate().millisecondsSinceEpoch;
    //   timeFilterSelect!(_res);
    // }
    // debugPrint("=====$_beginTime========$_endTime======");
  }

  @override
  _VTimeFilterState createState() => _VTimeFilterState();
}

class _VTimeFilterState extends State<VTimeFilter> {
  final DropDownMenuController _menuController = DropDownMenuController();

  String _beginTime = "";
  String _endTime = "";

  @override
  Widget build(BuildContext context) {
    return Flexible(
      // height: MediaQuery.of(context).size.height,
      child: Column(
        children: [
          Container(
            color: ColorsUtil.hex2Color(widget.filterBgColor!),
            //filter整体的背景色，必须的设置
            child: Row(
              children: [
                DropDownMenuHeader(
                    menuController: _menuController,
                    titles: [widget._pickerTitle]),
                Expanded(
                    child: VSearchBar(
                      text: _menuController.searchKeywords,
                      bgColor: widget.filterBgColor!,
                      searchBgColor: "#ffffff",
                      cursorColor: widget.cursorColor,
                      hintText: widget.searchHint,
                      maxLength: widget.searchLength,
                      height: 60,
                      inputCallBack: (value) {
                        debugPrint("========inputCallBack===========$value==");
                        setState(() {
                          _menuController.searchKeywords =
                              value; // _requestKeywordList(value);
                        });
                      },
                      inputCompletionCallBack: (value, isSubmitted) {
                        debugPrint(
                            "========inputCompletionCallBack===========$value========$isSubmitted======");
                        if (isSubmitted) {
                          _handleCallback(true, true);
                        }
                      },
                    ))
              ],
            ),
          ),
          Visibility(
            // visible: widget.daySelect![widget.selectDayIndex]["value"] == 10086,
            visible: widget.daySelect![_menuController.timeDropIndex]
            ["value"] ==
                10086,
            child: Container(
              color: ColorsUtil.hex2Color(widget.customBgColor ?? "#f3f4f5"),
              width: MediaQuery.of(context).size.width,
              padding: const EdgeInsets.only(bottom: 10, left: 5),
              child: Row(
                children: [
                  _createTimeWidget(_beginTime, 0),
                  SizedBox(
                    width: 25,
                    child: Text(
                      "至",
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        fontSize: 12,
                        color: ColorsUtil.hex2Color("#333333"),
                      ),
                    ),
                  ),
                  _createTimeWidget(_endTime, 1),
                  InkWell(
                    onTap: () {
                      DayUnits us = DayUnits.s;
                      if (widget.format == TimeFormat.formats["format2"]) {
                        us = DayUnits.m;
                      } else {
                        us = DayUnits.D;
                      }
                      int gap =
                          DayDart(_beginTime).toDate().millisecondsSinceEpoch -
                              DayDart(_endTime).toDate().millisecondsSinceEpoch;
                      if (gap > 0) {
                        // ToastUtils.toast("开始时间必须小于结束时间");
                        widget.checkCallBack("开始时间必须小于结束时间");
                      } else {
                        _handleCallback(true, true);
                      }
                    },
                    child: Container(
                      width: 40,
                      margin: const EdgeInsets.only(right: 5, left: 5),
                      height: 30,
                      decoration: const BoxDecoration(
                          color: Colors.deepPurple,
                          borderRadius: BorderRadius.all(Radius.circular(5))),
                      child: const Center(
                        child: Text(
                          "查询",
                          style: TextStyle(color: Colors.white, fontSize: 12),
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
          Expanded(
              child: Stack(children: [
                ...?widget.children,
                DropDownMenu(
                    height: _pickerItemHeight * widget.daySelect!.length +
                        (widget.daySelect!.length),
                    milliseconds: 300,
                    menuController: _menuController,
                    children: [
                      ListView.separated(
                        itemBuilder: (BuildContext context, int index) {
                          return Container(
                              color: Colors.white,
                              height: _pickerItemHeight,
                              padding: const EdgeInsets.only(left: 10, right: 10),
                              child: InkWell(
                                onTap: () {
                                  int currentIndex = _menuController.timeDropIndex;
                                  _menuController.changeTitle(
                                      0, widget.daySelect![index]["text"]);

                                  _menuController.changeTimeDrop(index);

                                  // setState(() {
                                  widget.selectDayIndex = index;
                                  bool isCustomDate =
                                      widget.daySelect![index]["value"] == 10086;
                                  if (!isCustomDate) {
                                    String format=widget.format!;
                                    if(widget.format!=TimeFormat.formats["format3"]){
                                      format=TimeFormat.formats["format3"];
                                    }

                                    _endTime = DayDart().format(format);
                                    int gap = widget
                                        .daySelect[widget.selectDayIndex]["value"];
                                    int gapDP = gap - 1;
                                    _beginTime = DayDart()
                                        .subtract(gapDP, DayUnits.D)
                                        .format(format);

                                    if (gapDP != 1) {

                                      _endTime = "$_endTime 23:59:59";
                                    } else {

                                      _endTime = "$_beginTime 23:59:59";
                                    }

                                    _beginTime += " 00:00:00";


                                    _handleCallback(!isCustomDate, isCustomDate);
                                  }

                                  // });
                                  if (isCustomDate ||
                                      widget.daySelect![currentIndex]["value"] ==
                                          10086) {
                                    //组件整理的时候吧这个更新放到这的（注释这个）  因为在组件库发现上面的有问题：选择后不能把选项给选中
                                    // setState(() {});
                                  }
                                  //组件整理的时候吧这个更新放到这的 新家这个 因为在组件库发现上面的有问题：选择后不能把选项给选中
                                  setState(() {});
                                },
                                child: Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    Text(
                                      widget.daySelect![index]["text"],
                                      style: TextStyle(
                                          color:
                                          _menuController.timeDropIndex == index
                                              ? ColorsUtil.hex2Color("#6c69ff")
                                              : Colors.black),
                                    ),
                                    Icon(
                                      Icons.check,
                                      size: 25,
                                      color: _menuController.timeDropIndex == index
                                          ? ColorsUtil.hex2Color("#6c69ff")
                                          : Colors.white,
                                    )
                                  ],
                                ),
                              ));
                        },
                        separatorBuilder: (BuildContext context, int index) {
                          return Divider(
                            height: 1,
                            color: ColorsUtil.hex2Color("#dcdcdc"),
                            thickness: 1,
                          );
                        },
                        itemCount: widget.daySelect!.length,
                      )
                    ]),
              ])),
        ],
      ),
    );
  }

  _handleCallback(bool isAttached, bool isCustomDate) {
    _res["beginTime"] = _beginTime;
    _res["endTime"] = _endTime;
    _res["filterKeywords"] = _menuController.searchKeywords;
    _res["isAttached"] = isAttached;
    _res["isCustomDate"] = isCustomDate;
    _res["endTimeMill"] = DayDart(_endTime).toDate().millisecondsSinceEpoch;
    _res["beginTimeMill"] = DayDart(_beginTime).toDate().millisecondsSinceEpoch;

    _res["selectDayIndex"] = widget.selectDayIndex;

    _res["isInit"] = false;

    if (widget.timeFilterSelect != null) {
      widget.timeFilterSelect!(_res);
    }
  }

  _createTimeWidget(String timeStr, int type) {
    return Expanded(
        flex: 1,
        child: InkWell(
          onTap: () {
            VPickerTool.showDatePicker(context,
                title: '请选择时间',
                dateType: widget._pickerType,
                // minTime: DateTime(int.parse(VTimeUtils.getYear()) - 1, 10, 10),
                // maxTime: DateTime(int.parse(VTimeUtils.getYear()) + 1, 10, 10),
                selectTime: DayDart(timeStr).toDate(),
                clickCallBack: (selectValue, selectIndexArr) {
                  // debugPrint(selectIndexArr);
                  // debugPrint(selectValue);
                  if (type == 0) {
                    _beginTime = selectValue;
                  } else {
                    _endTime = selectValue;
                  }

                  setState(() {});
                });
          },
          child: Container(
            height: _customElementHeight,
            decoration: BoxDecoration(
              // color: Colors.white,
                borderRadius: BorderRadius.circular(5),
                border: Border.all(
                  color: ColorsUtil.hex2Color("#dcdcdc"),
                )),
            child: Center(
              child: Text(
                timeStr,
                style: TextStyle(
                    color: ColorsUtil.hex2Color(
                      "#333333",
                    ),
                    fontSize: 13),
              ),
            ),
          ),
        ));
  }

  @override
  void initState() {
    if (widget.timeFilterSelect != null) {
      String format=widget.format!;
      // if(widget.format!=TimeFormat.formats["format3"]){
      //   format=TimeFormat.formats["format3"];
      // }

      _endTime = DayDart().format(format);
      int gap = widget.daySelect[widget.selectDayIndex]["value"];
      int gapDP = gap - 1;
      _beginTime = DayDart().subtract(gapDP, DayUnits.D).format(format);

      _res["beginTime"] = "${_beginTime.substring(0,10)} 00:00:00";
      _res["endTime"] = "${_endTime.substring(0,10)} 23:59:59";
      _res["filterKeywords"] = _menuController.searchKeywords;
      _res["endTimeMill"] = DayDart(_res["endTime"]).toDate().millisecondsSinceEpoch;
      _res["beginTimeMill"] =
          DayDart(_res["beginTime"]).toDate().millisecondsSinceEpoch;
      _res["isInit"] = true; //返回这个是为了   解决tab页面的切换  与tab初始化的不能区分问题   就是为了mvvm使用

      // _res["selectDayIndex"]=widget.selectDayIndex;

      _menuController.timeDropIndex = widget.selectDayIndex;

      widget.timeFilterSelect!(_res);
      setState(() {});
    }
  }
}
