/// @author:   wzp_vicky
/// @time:     2022年11月8日09:49:21
/// @version:  V1.1
/// @description:底部选择器 包含日期，单列、多列文本
///              这个需要依赖https://pub.flutter-io.cn/packages/flutter_picker
///
import 'package:flutter/material.dart';
import 'package:flutter_picker/flutter_picker.dart';

import '../../utils/colors_util.dart';
import '../search/VSearch.dart';

const String _titleNormalText = '请选择'; //弹框中间的标题
const double _headerLineHeight = 1; //标题下的分割线
Color _headerLineColor = ColorsUtil.hex2Color("#dcdcdc"); //标题下的分割线
const String _cancelText = '取消';
const String _confirmText = '确定';
const String _yearSuffix = '年';
const String _monthSuffix = '月';
const String _daySuffix = '日';
const List<String> _strAMPM = ['上午', '下午'];
const double _PickerHeight = 200; //pickerview内容区的高度
const double _PickerItemHeight = 50.0; //每一项的高度，

const double _dialogTopRadius = 10.0; //圆角的大小
const double _selectTextFontSize = 20.0;

/// 选择回调
/// 单列选择器返回选中行对象和index
/// 多列选择器返回选中行对象数组和index数组
/// 时间选择器返回选中行时间（时间格式：2022-07-03 15:00:46）和index数组
typedef _ClickCallBack = void Function(
    dynamic selectValue, dynamic selectIndexArr);

enum PickerDateType {
  YMD, // y, m, d
  YM, // y ,m
  YMD_HM, // y, m, d, hh, mm
  YMD_HMs, // y, m, d, hh, mm  ss
  YMD_AP_HM, // y, m, d, ap, hh, mm
}

enum PickerType {
  String,
  Array,
  Date,
}

class VPickerTool {
  /// 单列
  static void showStringPicker<T>(
      BuildContext context, {
        required List data,
        String? title,
        String? labelKey, // 对象数组的文字字段
        int selectIndex = 0,
        Widget? contentOtherChild,
        @required _ClickCallBack? clickCallBack,
      }) {
    if (data.length <= 0) {
      return;
    }
    _showPicker(
      context,
      data: data,
      title: title,
      selecteds: [selectIndex],
      pickerType: PickerType.String,
      // contentOtherChild: contentOtherChild,
      adapter: labelKey != null
          ? PickerDataAdapter(pickerData: data.map((e) => e[labelKey]).toList())
          : PickerDataAdapter(pickerData: data),
      clickCallBack: clickCallBack,
    );
  }

  /// 多列
  static void showArrayPicker<T>(
      BuildContext context, {
        required List data,
        String? title,
        String? labelKey, // 对象数组的文字字段
        List<int>? selectIndex,
        required _ClickCallBack clickCallBack,
      }) {
    if (data.length <= 0) {
      return;
    }
    _showPicker(
      context,
      data: data,
      title: title,
      selecteds: selectIndex,
      pickerType: PickerType.Array,
      adapter: labelKey != null
          ? PickerDataAdapter(
          pickerData: data.map((e) {
            return e.map((e2) => e2[labelKey]).toList();
          }).toList(),
          isArray: true)
          : PickerDataAdapter(pickerData: data, isArray: true),
      clickCallBack: clickCallBack,
    );
  }

  /// 日期选择器
  static void showDatePicker(
      BuildContext context, {
        String? title,
        PickerDateType? dateType,
        DateTime? maxTime,
        DateTime? minTime,
        DateTime? selectTime,
        int? yearBegin = 1900,
        int? yearEnd = 2100,
        int? minHour = 0,
        int? maxHour = 23,
        required _ClickCallBack clickCallBack,
      }) {
    int timeType;
    if (dateType == PickerDateType.YM) {
      timeType = PickerDateTimeType.kYM;
    } else if (dateType == PickerDateType.YMD_HM) {
      timeType = PickerDateTimeType.kYMDHM;
    } else if (dateType == PickerDateType.YMD_AP_HM) {
      timeType = PickerDateTimeType.kYMD_AP_HM;
    } else if (dateType == PickerDateType.YMD_HMs) {
      timeType = PickerDateTimeType.kYMDHMS;
    } else {
      timeType = PickerDateTimeType.kYMD;
    }

    _showPicker(
      context,
      title: title,
      pickerType: PickerType.Date,
      adapter: DateTimePickerAdapter(
        type: timeType,
        isNumberMonth: true,
        yearSuffix: _yearSuffix,
        monthSuffix: _monthSuffix,
        daySuffix: _daySuffix,
        strAMPM: _strAMPM,
        maxValue: maxTime,
        minValue: minTime,
        value: selectTime ?? DateTime.now(),
        minHour: minHour,
        maxHour: maxHour,
        yearBegin: yearBegin,
        yearEnd: yearEnd,
      ),
      clickCallBack: clickCallBack,
    );
  }
}

// ---------------------------------------------------------------------------------------------------

/// 自定义picker
_showPicker(
    context, {
      List? data,
      String? dialogHeaderBgColor, //弹框顶部的背景颜色  也就是标题栏的背景颜色

      String? title, //标题文字
      String? titleColor,
      double titleFontSize = 16,
      String cancelTxt = _cancelText,
      String cancelColor = "#999999",
      double cancelSize = 14,
      String confirmTxt = _confirmText,
      String confirmColor = "#333333",
      double confirmSize = 14,
      List<int>? selecteds,
      PickerType? pickerType,
      required PickerAdapter adapter,
      _ClickCallBack? clickCallBack,
      String selectAreaBgColor = "#ffffff",
      String? selectLineColor, //picker上线边线的颜色
      String? selectTxtColor,
      double selectTxtFontSize = 15,
      // Widget? contentOtherChild
    }) {
  var picker = Picker(
      adapter: adapter,
      selecteds: selecteds,
      //选中哪个
      height: _PickerHeight,
      itemExtent: _PickerItemHeight,
      //标题设置
      title: Text(title ?? _titleNormalText,
          style: TextStyle(
              color: ColorsUtil.hex2Color(titleColor ?? "#333333"),
              fontSize: titleFontSize)),

      // 取消按钮设置
      cancelText: cancelTxt,
      cancelTextStyle: TextStyle(
          color: ColorsUtil.hex2Color(cancelColor), fontSize: cancelSize),

      // 确定按钮
      confirmText: confirmTxt,
      confirmTextStyle: TextStyle(
          color: ColorsUtil.hex2Color(confirmColor), fontSize: confirmSize),

      // contentWidget:  contentOtherChild,



      //标题下分割线
      headerDecoration: BoxDecoration(
        border: Border(
          bottom: BorderSide(color: _headerLineColor, width: _headerLineHeight),
        ),
      ),
// 请去平台修正运单   开始tiem  为空  “运单未进厂”
      //中间的选择区域配置（pick及其周边）
      selectionOverlay: Column(
        children: [
          Container(
            height: _PickerItemHeight,
            decoration: BoxDecoration(
                color: Colors.transparent,
                border: Border(
                    top: BorderSide(
                        color:
                        ColorsUtil.hex2Color(selectLineColor ?? '#dcdcdc'),
                        width: 1),
                    bottom: BorderSide(
                        color:
                        ColorsUtil.hex2Color(selectLineColor ?? '#dcdcdc'),
                        width: 1))),
          )
        ],
      ),
      //整个选择区域的颜色
      backgroundColor: ColorsUtil.hex2Color(selectAreaBgColor),
      textAlign: TextAlign.left,

      //中间的选择项的配置
      selectedTextStyle: TextStyle(
          color: ColorsUtil.hex2Color(selectTxtColor ?? '#333333'),
          fontSize: selectTxtFontSize),
      textStyle: TextStyle(
        fontSize: selectTxtFontSize - 2,
        color: ColorsUtil.hex2Color(selectTxtColor ?? '#333333'),
      ),
      onConfirm: (Picker picker, List selectIndexArr) {
        if (pickerType == PickerType.String) {
          var selectIndex = selectIndexArr[0];
          clickCallBack?.call(data![selectIndex], selectIndex);
        }
        if (pickerType == PickerType.Array) {
          var selectItemArr = [];
          for (int i = 0; i < selectIndexArr.length; i++) {
            int j = selectIndexArr[i];
            selectItemArr.add(data![i][j]);
          }
          clickCallBack?.call(selectItemArr, selectIndexArr);
        }
        if (pickerType == PickerType.Date) {
          // var time = (picker.adapter as DateTimePickerAdapter).value;
          clickCallBack?.call(
              picker.adapter.text.split('.')[0], selectIndexArr);
        }
      });

  picker.showModal(context, backgroundColor: Colors.transparent,
      isScrollControlled: true,  builder: (context, view) {
        return Material(
          color: ColorsUtil.hex2Color(dialogHeaderBgColor ?? '#ffffff'),
          borderRadius: const BorderRadius.only(
              topLeft: Radius.circular(_dialogTopRadius),
              topRight: Radius.circular(_dialogTopRadius)),
          child: Container(
            // padding: EdgeInsets.only(top: 15, bottom: 0),
            child: view,
          ),);
      });
}
