import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'VCascadePickerView.dart';

///
/// @author:   wzp_vicky
/// @time:     2022年11月7日23:40:04
/// @version:  V1.1
/// @description: 简单来个 选择  级联选择器（多维数组结构数据）

const String _labelKey = 'label';
const String _titleText = '请选择';
const String _tabText = '请选择';

const double _headerRadius = 10.0;//圆角弹框默认圆角值


/// 选择回调，返回选中对象
typedef _ClickCallBack = void Function(dynamic selectValue);

class VCascadePicker {
  static void show(
    BuildContext context, {
    required List data, // 多维数组，根据维数决定列数
    String labelKey = _labelKey, // 对象数组的文字字段
    String title = _titleText,//弹出框中间的提示信息
    String tabText = _tabText,
    bool isShowRadius = true,//是否是圆角的弹框
    _ClickCallBack? clickCallBack,
  }) {
    if (data.length <= 0) {
      return;
    }
    var _radius = isShowRadius ? _headerRadius : 0.0;

    showModalBottomSheet<void>(
      context: context,
      isScrollControlled: true, // 使用true则高度不受16分之9的最高限制
      // 设置圆角
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(_radius),
          topRight: Radius.circular(_radius),
        ),
      ),
      // 抗锯齿
      clipBehavior: Clip.antiAlias,
      builder: (BuildContext context) {
        return SafeArea(
          child: VCascadePickerView(
            data: data,
            labelKey: labelKey,
            title: title,
            tabText: tabText,
            clickCallBack: clickCallBack,
          ),
        );
      },
    );
  }
}



