/// @author:   wzp_vicky
/// @time:     2022年11月15日18:44:10
/// @version:  V1.1
/// @description:popumenu  简单的工具   等闲了做公共的第二版
///

import 'package:flutter/material.dart';
import 'dart:ui' as ui show window;

import '../../utils/colors_util.dart';
import '../../utils/screen_utils.dart';


List _listData = [
  {'text': '卫星', 'icon': ""},
  {'text': '路况', 'icon': ""},
];

const Color _bgColor = Color(0xFF2D2D2D);
const double _fontSize = 16.0;
const double _cellHeight = 50.0;
const double _imgWH = 22.0;

typedef _ClickCallBack = void Function(int selectIndex, String selectText);

class VPopMenus {
  /// 显示pop
  /// top  这距离不包含导航栏和状态栏
  static void show(BuildContext context,
      {_ClickCallBack? clickCallback,
      Widget? popuContentWeight,
      double? topdp,
      double? leftdp,
      double? bottomdp ,
      double? rightDP }) {
    // Cell
    Widget _buildMenuCell(dataArr) {
      return ListView.builder(
          itemCount: dataArr.length,
          itemExtent: _cellHeight,
          padding: const EdgeInsets.all(0.0),
          physics: const NeverScrollableScrollPhysics(),
          itemBuilder: (BuildContext context, int index) {
            return Material(
                color: ColorsUtil.hex2Color("#ffffff", alpha: 0.8),
                // color:_bgColor,
                child: InkWell(
                    onTap: () {
                      Navigator.pop(context);
                      if (clickCallback != null) {
                        clickCallback(index, _listData[index]['text']);
                      }
                    },
                    child: Row(
                      children: <Widget>[
                        const SizedBox(width: 25),
                        Image.asset(dataArr[index]['icon'],
                            width: _imgWH, height: _imgWH, color: Colors.white),
                        const SizedBox(width: 15),
                        Text(dataArr[index]['text'],
                            style: const TextStyle(
                                color: Colors.black, fontSize: _fontSize)),
                      ],
                    )));
          });
    }

    Widget _menusView(dataArr) {
      var cellH = dataArr.length * _cellHeight;
      var navH = VScreenUtils.navigationBarHeight;
      return Positioned(
        right: rightDP,
        bottom: bottomdp,
        left: leftdp,
        top: (topdp != null ? navH + topdp! : topdp),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.end,
          children: <Widget>[
            // Image.asset(AssetsRes.TAB_H_T, width: 28, height: 5),
            ClipRRect(
                borderRadius: BorderRadius.circular(5),

                child: Container(
                    // color: _bgColor,
                    // width: 160,
                    // height: cellH,
                    child: popuContentWeight ?? _buildMenuCell(dataArr)))
          ],
        ),
      );
    }

    Navigator.of(context)
        .push(DialogRouter(_BasePopMenus(child: _menusView(_listData))));
  }

  /// 显示带线带背景 pop
  static void showLinePop(BuildContext context,
      {bool isShowBg = false, _ClickCallBack? clickCallback}) {
    // 带线
    Widget _buildMenuLineCell(dataArr) {
      return ListView.separated(
        itemCount: dataArr.length,
        padding: const EdgeInsets.all(0.0),
        physics: const NeverScrollableScrollPhysics(),
        itemBuilder: (BuildContext context, int index) {
          return Material(
              color: _bgColor,
              child: InkWell(
                  onTap: () {
                    Navigator.pop(context);
                    if (clickCallback != null) {
                      clickCallback(index, _listData[index]['text']);
                    }
                  },
                  child: SizedBox(
                    height: _cellHeight,
                    child: Row(
                      children: <Widget>[
                        const SizedBox(width: 25),
                        Image.asset(dataArr[index]['icon'],
                            width: _imgWH, height: _imgWH, color: Colors.white),
                        const SizedBox(width: 12),
                        Text(dataArr[index]['text'],
                            style: const TextStyle(
                                color: Colors.white, fontSize: _fontSize))
                      ],
                    ),
                  )));
        },
        separatorBuilder: (context, index) {
          return const Divider(
            height: .1,
            indent: 50,
            endIndent: 0,
            color: Color(0xFFE6E6E6),
          );
        },
      );
    }

    Widget _menusView(dataArr) {
      var cellH = dataArr.length * _cellHeight;
      var navH = VScreenUtils.navigationBarHeight;
      if (isShowBg == true) {
        navH = navH - VScreenUtils.topSafeHeight;
      } else {
        navH = navH - 10;
      }
      return Positioned(
        right: 10,
        top: navH,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.end,
          children: <Widget>[
            Image.asset('assets/images/popMenus/ic_menu_up_arrow.png',
                width: 28, height: 5),
            ClipRRect(
                borderRadius: BorderRadius.circular(5),
                child: Container(
                    color: _bgColor,
                    width: 160,
                    height: cellH,
                    child: _buildMenuLineCell(dataArr)))
          ],
        ),
      );
    }

    if (isShowBg == true) {
      // 带背景
      showDialog(
          context: context,
          barrierDismissible: false,
          builder: (context) {
            return _BasePopMenus(child: _menusView(_listData));
          });
    } else {
      Navigator.of(context)
          .push(DialogRouter(_BasePopMenus(child: _menusView(_listData))));
    }
  }
}

class _BasePopMenus extends Dialog {
  _BasePopMenus({
    Key? key,
    this.child,
  }) : super(key: key);

  final Widget? child;

  @override
  Widget build(BuildContext context) {
    return Material(
      type: MaterialType.transparency,
      child: Stack(
        fit: StackFit.expand,
        children: <Widget>[
          GestureDetector(onTap: () => Navigator.pop(context)),
          // 内容
          child ?? Container()
        ],
      ),
    );
  }
}

class DialogRouter extends PageRouteBuilder {
  final Widget page;

  DialogRouter(this.page)
      : super(
          opaque: false,
          // 自定义遮罩颜色
          barrierColor: Colors.white10.withAlpha(1),
          transitionDuration: const Duration(milliseconds: 150),
          pageBuilder: (context, animation, secondaryAnimation) => page,
          transitionsBuilder: (context, animation, secondaryAnimation, child) =>
              child,
        );
}

class CustomDialog extends Dialog {
  const CustomDialog({
    Key? key,
    this.child,
    this.clickBgHidden = false, // 点击背景隐藏，默认不隐藏
  }) : super(key: key);

  final Widget? child;
  final bool clickBgHidden;

  @override
  Widget build(BuildContext context) {
    return Material(
        // 透明层
        type: MaterialType.transparency,
        child: Stack(
          children: <Widget>[
            InkWell(
              onTap: () {
                if (clickBgHidden == true) {
                  Navigator.pop(context);
                }
              },
              child: SizedBox(
                width: MediaQuery.of(context).size.width,
                height: MediaQuery.of(context).size.height,
              ),
            ),
            // 内容
            Center(child: child)
          ],
        ));
  }

  double get navigationBarHeight {
    MediaQueryData mediaQuery = MediaQueryData.fromWindow(ui.window);
    return mediaQuery.padding.top + kToolbarHeight;
  }
}
