///
/// @author:   wzp_vicky
/// @time:     2022年11月3日00:10:39
/// @version:  V1.1
/// @description: easyrefresh的封装、也使用了空页面组件
///     easy_refresh:
///           文档：https://github.com/xuelongqy/flutter_easy_refresh/blob/v3/README_CN.md
///                 https://pub.flutter-io.cn/packages/easy_refresh
///
/// @author:   wzp_vicky
/// @time:     2022年11月21日10:47:09
/// @version:  V1.2
/// @description: 之前做海外项目时遇到过for-each一把发出去好多请求，请求会出现异常的问题，当时任务是dart单线程语言的问题
/// 当时改成队列的形式去发送请求；但是在做【伊利奶粉】时
///
/// @author:   wzp_vicky
///  @time:    2023年9月11日07:54:42
///  @version:  V1.3
///  @description: 在引入easy_refresh: ^3.3.2+1时提供了新属性：refreshOnStart，所以不再需要之前的初始化时写的：
///                   Future.delayed(const Duration(milliseconds: 500), () {
///                       _controller.callRefresh();
///                   })

import 'dart:async';
import 'dart:ffi';
import 'package:flutter/material.dart';
import 'package:easy_refresh/easy_refresh.dart';
import 'package:v_base_framework/http/bean/AppException.dart';
import 'package:v_base_framework/mixin/page_mixin.dart';

import '../empty/empty_widget.dart';

typedef Success<T> = Function(T data);
typedef Fail = Function(int code, String msg);

// /// 父组件中调用 globalKeyRefresh.currentState.request2GetListData()
// /// 初始化调用要延时加载
GlobalKey<BaseRefreshViewState> refreshGlobalKey = GlobalKey();
//这个东西如果在tabview中使用：建议在tab页面自行创建，若果只在一个页面可以使用这个

class VRefreshView extends StatefulWidget {
  final List data; // 数据
  // final int limit; // 分页条数，为了控制上拉加载完成显示状态
  // final bool laodAll; // ，为了控制上拉加载完成显示状态  再次改版后  暂定为设置一进来不允许上啦的场景
  final IndexedWidgetBuilder? itemBuilder; // listview子组件内的itemBuilder
  final IndexedWidgetBuilder? separatorBuilder; // listview子组件内的separatorBuilder 分割线
  final FutureOr Function()? onRefresh; // 刷新回调(null为不开启下拉刷新)
  final FutureOr Function()? onLoad; // 加载回调(null为不开启上拉加载)
  final bool firstRefresh; // 首次刷新
  final Header? header; // 不传使用默认header
  final Footer? footer; // 不传使用默认footer
  final Widget? child; // 自定义子组件, 优先级高于itemBuilder,使用后itemBuilder失效
  final Widget? empty; // 空视图
  final EasyRefreshController? controller; // EasyRefresh controller
  const VRefreshView({
    Key? key,
    required this.data,
    // this.limit = 15,
    // this.laodAll = false,
    this.itemBuilder,
    this.separatorBuilder,
    this.onRefresh,
    this.onLoad,
    this.firstRefresh = false,
    this.header,
    this.footer,
    this.child,
    this.empty,
    this.controller,
  }) : super(key: key);

  @override
  State<VRefreshView> createState() => BaseRefreshViewState();
}

class BaseRefreshViewState<T extends VRefreshView> extends State<T> with PageMixins {
  bool _isNetWorkError = false;
  late EasyRefreshController _controller;
  final GlobalKey _key1 = GlobalKey();
  double? _easyRefreshHeight;
  List _dataArr = [];

  bool hasMore = false;
  int _pageIndex = 0;
  final int _pageSize = 20;

  @override
  void initState() {
    debugPrint("==================一个很牛逼的分页组件正在向你迎面走来===============");
    super.initState();

    _controller = widget.controller ??
        EasyRefreshController(
          controlFinishRefresh: true,
          controlFinishLoad: true,
        );
    // Deprecated  easy_refresh: ^3.3.2+1增加了refreshOnStart就不在需要这个
    // Future.delayed(const Duration(milliseconds: 500), () {
    //   _controller.callRefresh();
    // });
  }

  @override
  void dispose() {
    debugPrint("==================一个很牛逼的分页组件走了==============");
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    //监听绘制完成

    WidgetsBinding.instance.addPostFrameCallback((duration) {
      RenderBox? box = _key1.currentContext?.findRenderObject() as RenderBox?;
      _easyRefreshHeight = box!.size.height;
      setState(() {});
    });
    return EasyRefresh(
      key: _key1,
      controller: _controller,
      canLoadAfterNoMore: hasMore,
      header: widget.header ?? _defaultHeader(),
      footer: widget.footer ?? _defaultFooter(),
      refreshOnStart: widget.firstRefresh,
      onRefresh: widget.onRefresh,
      onLoad: widget.data.isEmpty ? null : widget.onLoad,
      child: widget.child ?? _defaultChild(),
    );
  }

  _defaultChild() {
    if (widget.data.isEmpty) {
      return ListView.builder(
        itemCount: 1,
        itemBuilder: (context, index) => Container(
          height: _easyRefreshHeight ?? 0,
          child: widget.empty ?? _emptyWidget(),
        ),
      );
    } else {
      return ListView.separated(
        itemCount: widget.data.length,
        itemBuilder: widget.itemBuilder!,
        separatorBuilder: widget.separatorBuilder!,
      );
    }
  }

  _defaultHeader() {
    return const ClassicHeader(
      dragText: '下拉刷新',
      armedText: '释放刷新',
      readyText: '加载中...',
      processingText: '加载中...',
      processedText: '加载完成',
      noMoreText: '没有更多',
      failedText: '加载失败',
      messageText: '最后更新于 %T',
    );
  }

  _defaultFooter() {
    return const ClassicFooter(
      dragText: '上拉加载',
      armedText: '释放刷新',
      readyText: '加载中...',
      processingText: '加载中...',
      processedText: '加载完成',
      noMoreText: '没有更多',
      failedText: '加载失败',
      messageText: '最后更新于 %T',
      showMessage: false, // 隐藏更新时间
    );
  }

  _emptyWidget() {
    if (_isNetWorkError) {
      return VEmptyView(
        type: EmptyType.error,
        clickCallBack: () async => _controller.callRefresh(),
      );
    }
    return const VEmptyView();
  }

  callRefresh() {
    if (_controller != null) {
      // ToastUtils.toast("msg tab刷新");
      _controller.callRefresh();
    }
  }

  //谨记：params的其他参数搞成dynamic   否则分页的参数加进去会报错

  request2GetListData(BuildContext context, String url,
      {String? loadingText, Map<String, dynamic>? params, isLoadMore = false, String pagingKey = "pageParamVo", Success? success, Fail? fail}) async {
    // String requestUrl = HttpUrlHelper.getRequestUrl(Api.mine);
    _pageIndex = hasMore && isLoadMore ? _pageIndex + 1 : 1;

    if (params != null && pagingKey.isNotEmpty) {
      dynamic pageParams = {
        'curPage': _pageIndex,
        'pageSize': _pageSize,
      };
      try {
        params![pagingKey] = pageParams;
      } catch (msg) {
        debugPrint(msg.toString());
      }
    }

    try {
      dynamic res = await request2Post(url, context, data: params, isShowLoading: false);

      var tempData ;
      if (res is List) {
        hasMore = false;
        tempData = res;
      } else {
        int totalRecordsCount = res["totalRecords"] ?? 0;
        int pageIndex = res["curPage"] ?? 0;
        int pageSize = res["pageSize"] ?? 0;

        hasMore = pageIndex * pageSize < totalRecordsCount;
        tempData = res["data"];
      }

      if (isLoadMore) {
        _dataArr.addAll(tempData);
      } else {
        _dataArr = tempData;
      }

      setState(() {
        _isNetWorkError = false;
      });
      success?.call(_dataArr);

      _handleRefresh(isLoadMore);
    } on AppException catch (e) {
      setState(() {
        _isNetWorkError = true;
      });
      _handleRefresh(isLoadMore);
      fail?.call(-1008611, e.getMessage());
    }
  }

  void _handleRefresh(isLoadMore) {
    // if (_controller == null) {
    if (isLoadMore) {
      if (widget.onLoad != null) {
        _controller.finishLoad(hasMore ? IndicatorResult.success : IndicatorResult.noMore);
      } else {}
    } else {
      // if (widget.onRefresh != null) {
      //   _controller.finishRefresh();
      // }

      // Future.delayed(const Duration(milliseconds: 500), () {
      _controller.finishRefresh();
      // });

      if (widget.onLoad != null) {
        _controller.resetFooter();
      }

      if (!hasMore) {
        _controller.finishLoad(IndicatorResult.noMore);
      }
    }
    // }
  }
}
