import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import '../../utils/colors_util.dart';
import '../field/field2_widget.dart';

///
/// @author:   wzp_vicky
/// @time:     2022年11月5日22:36:37
/// @version:  V1.1
/// @description: 搜索框组件   查看了下大多数app是没有右边的搜索按钮的，所以我这里也不要了；至于像微信、uc等右边显示取消的场景，这里暂时也不支持
///
const double _searchViewHeight = 50.0;
const String _hintText = '请输入';
const double _borderRadius = 5.0;
const double _borderWidth = 3;
// const EdgeInsetsGeometry _contentPadding = EdgeInsets.symmetric(vertical: 4);

typedef _InputCallBack = void Function(String value);
typedef _InputCompletionCallBack = void Function(
    String value, bool isSubmitted);

class VSearchBar extends StatelessWidget {
  final double height;
  final String? text;
  final String hintText;
  final int maxLength; // 最大长度，默认15
  final String? bgColor; //整体的背景色
  final String? searchBgColor; //搜索的背景色
  final String? searchBorderColor; //搜索的边框色
  final bool isShowDeleteBtn; // 是否显示右侧删除按钮，默认显示
  final bool isShowBorder; // 是否显示边框，默认不显示
  // final EdgeInsetsGeometry
  //     contentPadding; //内边距 当父组件固定高度时，文本一行显示文本过多会出现文字显示不全bug,可设置EdgeInsets.symmetric(vertical: 4)
  final TextInputAction? textInputAction; // 键盘右下角按钮类型
  final _InputCallBack? inputCallBack;
  final _InputCompletionCallBack? inputCompletionCallBack;
  final Widget? searchImg;
  final String? hintColor;
  final String? cursorColor;
  final EdgeInsetsGeometry? padding;

  const VSearchBar(
      {Key? key,
      this.height = _searchViewHeight,
      this.text = '',
      this.hintText = _hintText,
      this.maxLength = 15,
      this.bgColor,
      this.isShowDeleteBtn = true,
      this.isShowBorder = false,
      // this.contentPadding = _contentPadding,
      this.textInputAction = TextInputAction.search,
      this.inputCallBack,
      this.inputCompletionCallBack,
      this.searchImg,
      this.hintColor,
      this.searchBorderColor,
      this.padding,
      this.cursorColor,
      this.searchBgColor})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    // 默认颜色
    // var searchBarBgColor = Colors.blue; //搜索框的背景色
    // var searchBarBorderColor = Colors.purple;

    return Container(
      color: ColorsUtil.hex2Color(bgColor ?? "#f3f4f5"),
      child: Container(
        decoration: BoxDecoration(
          color: ColorsUtil.hex2Color(searchBgColor??"#ffffff"),
          border: isShowBorder
              ? Border.all(
                  color: ColorsUtil.hex2Color(searchBorderColor ?? "#39aaf3"),
                  width: _borderWidth)
              : null,
          borderRadius: const BorderRadius.all(Radius.circular(_borderRadius)),
        ),

        alignment: Alignment.center,
        margin: this.padding ?? EdgeInsets.all(10),
        height: height - 20,
        // height: height +120,
        // padding: EdgeInsets.only( right: 15),
        child: VField2(
          isShowBorder: false,
          maxLength: maxLength,
          leftWidget: searchImg ??
              const Icon(
                Icons.search,
                size: 25,
                color: Colors.grey,
              ),
          text: text,
          hintText: hintText,
          hintColor: hintColor ?? "#dcdcdc",
          isShowDeleteBtn: isShowDeleteBtn,
          border: InputBorder.none,
          isDense: true,
          contentPadding: EdgeInsets.all(0),
          textInputAction: textInputAction,
          inputCallBack: inputCallBack,
          inputCompletionCallBack: inputCompletionCallBack,
          hintFontSize: 12,
          cursorColor: cursorColor ?? "#039aaf2",
        ),
      ),
    );
  }
}
