import 'package:flutter/material.dart';

import '../../utils/colors_util.dart';
import 'VStepperLeft.dart';

///
/// @author:   wzp_vicky
/// @time:     2022年11月4日23:30:48
/// @version:  V1.1
/// @description: 一个自定义的stepper组件。
///
///     首先：flutter提供了一个stepper组件，但是呢，个人觉得人家这个才是严格意义的stepper,而且呢
///           小程序现采用的有赞组件里面的stepper也跟flutter的很类似
///     但是易流的这种流程貌似更加适合这种，所以就仿照小程序现有的那组件做一个
///
/// 需要表示是不是高亮点  线的颜色修改
///

class VStepper extends StatelessWidget {
  late double titleLineHeight; //title前面的圆上下的高度，这个不包括内容区前面的线
  late double lineWidth; //线的宽度
  late double circleDiameter; //圆的直径
  late double circleMargin; //圆点距离左右的距离
  late double _stepMargin;
  late double _contentPaddingLeft;
  final bool isShowLastLine;
  final int stepCount;
  bool isOnlyLightCircle; //只点亮圈 默认是true

  // 所有的一样的时候  就把亮色和暗色设置成一样的
  late String normalLineColor;
  late String lightLineColor;
  int lightIndex; //点亮的下标  默认是0  -1的时候是都不点亮
  Function? createTitle;
  Function? createContent;

  late double titleFontSize;

  VStepper(
      {double? titleLineHeight,
        double? lineWidth,
        double? circleDiameter,
        double? circleMargin,
        String? normalLineColor,
        String? lightLineColor,
        this.isShowLastLine = false,
        this.isOnlyLightCircle = true,
        this.lightIndex = 0,
        this.createTitle,
        this.createContent,
        this.stepCount = 3}) {
    this.titleLineHeight = titleLineHeight ?? 32;
    this.lineWidth = lineWidth ?? 2;
    this.circleDiameter = circleDiameter ?? 16;
    this.circleMargin = circleMargin ?? 16;
    this.lightLineColor = lightLineColor ?? "#cd0000";
    this.normalLineColor = normalLineColor ?? "#d2d2d2";

    _stepMargin =
        this.circleDiameter / 2 + this.circleMargin - (this.lineWidth / 2);
    _contentPaddingLeft =
        (this.circleDiameter + this.circleMargin) / 2 + this.lineWidth;
  }

  @override
  Widget build(BuildContext context) {
    return ListView.builder(
      shrinkWrap: true,
      physics: const NeverScrollableScrollPhysics(),
      itemCount: stepCount,
      itemBuilder: (BuildContext context, int index) {
        return Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Container(
                  // 圆和线
                  height: titleLineHeight * 2,
                  // color: Colors.yellow,
                  child: LeftLineWidget(index != 0,
                      index == stepCount - 1 ? isShowLastLine : true,
                      //圈下面的短线现不现实
                      isLightCircle: index <= lightIndex,
                      //圈是否被点亮
                      isLightLine:
                      isOnlyLightCircle ? false : index <= lightIndex,
                      lineHeight: titleLineHeight,
                      circleMargin: circleMargin,
                      lineWidth: lineWidth,
                      normalLineColor: normalLineColor,
                      lightLineColor: lightLineColor,
                      circleDiameter: circleDiameter),
                ),
                Expanded(
                    child: Container(
                      // color: Colors.deepOrange,
                      height: titleLineHeight * 2,
                      alignment: Alignment.centerLeft,
                      child: createTitle == null
                          ? Text(
                        '这是一个牛逼的标题',
                        style: TextStyle(
                          fontSize: 13,
                        ),
                        overflow: TextOverflow.ellipsis,
                      )
                          : createTitle!(index),
                    ))
              ],
            ),
            Container(
              decoration: BoxDecoration(
                // color: Colors.orange,
                  border: Border(
                      left: BorderSide(
                          width: lineWidth, color: getLineColor(index)))),
              margin: EdgeInsets.only(left: _stepMargin), //23
              padding: EdgeInsets.fromLTRB(_contentPaddingLeft, 0, 15, 15),
              child: createContent == null
                  ? Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: const <Widget>[
                  Text('这是一段描述demo，正确使用请自定义',
                      style: TextStyle(
                        fontSize: 13,
                      )),
                  Text('时间：2022年11月5日16:43:38',
                      style: TextStyle(
                        fontSize: 9,
                      ))
                ],
              )
                  : createContent!(index),
            )
          ],
        );
      },
    );
  }

  //所有的一样的时候  就把亮色和暗色改成一样的
  getLineColor(int index) {
    if (index == stepCount - 1 && !isShowLastLine) {
      return Colors.transparent;
    } else if (isOnlyLightCircle) {
      //仅仅点亮圆
      return ColorsUtil.hex2Color(normalLineColor!);
    } else if (index <= lightIndex) {
      return ColorsUtil.hex2Color(lightLineColor!);
    } else {
      //只点亮圈或者其他的
      return ColorsUtil.hex2Color(normalLineColor!);
    }
  }
}
