import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import '../../utils/colors_util.dart';

///
/// @author:   wzp_vicky
/// @time:     2022年11月4日23:47:34
/// @version:  V1.1
/// @description: stepper左半部分
/// 左侧的圆和线是3个部分，分别是圆的上面那条线，和圆，以及圆下面的那条线，
///

class LeftLineWidget extends StatelessWidget {
  static const double _topHeight = 51; //圆上下的线高度
  static const double _lineWidth = 2; // 竖线的宽度

  final bool showTop; //上面那条线是否显示
  final bool showBottom; //下面那条线是否显示   不是最后那个item的线啊
  final double lineHeight; //线的高度
  final double circleDiameter; //圆的直径
  final double lineWidth; //线的宽度
  final double circleMargin; //圆点距离左右的距离
  final String? normalLineColor; //
  final String? lightLineColor;
  final bool isLightCircle; //点亮圈
  final bool isLightLine; //点亮线

  const LeftLineWidget(this.showTop, this.showBottom,
      {this.lineHeight = _topHeight,
      this.circleDiameter = 16,
      this.lineWidth = _lineWidth,
      this.normalLineColor,
      this.lightLineColor,
      this.isLightCircle = false,
      this.isLightLine = false,
      this.circleMargin = 16});

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.symmetric(horizontal: circleMargin), //圆和线的左右外边距
      // color: Colors.redAccent,
      width: circleDiameter,
      child: CustomPaint(
        painter: LeftLinePainter(showTop, showBottom,
            lineHeight: lineHeight,
            isLightCircle: isLightCircle,
            isLightLine:isLightLine,
            lineWidth: lineWidth,
            lightLineColor: lightLineColor ?? "#cd0000",
            normalLineColor: normalLineColor ?? "#dcdcdc"),
      ),
    );
  }
}

class LeftLinePainter extends CustomPainter {
  final bool showTop; //是否显示圆上面的线
  final bool showBottom; //是否显示圆下面的线
  final bool isLightCircle; //圆形是否点亮
  final bool isLightLine; //圆形是否点亮
  final double lineHeight; //线的高度
  final double lineWidth; //线的宽度
  final String lightLineColor;
  final String normalLineColor;

  const LeftLinePainter(
    this.showTop,
    this.showBottom, {
    required this.lineHeight,
    required this.lineWidth,
    this.isLightCircle = false,
    this.isLightLine = true,
    required this.normalLineColor,
    required this.lightLineColor,
  });

  @override
  void paint(Canvas canvas, Size size) {
    double centerX = size.width / 2; //容器X轴的中心点
    Paint linePain = Paint(); // 创建一个画线的画笔
    linePain.color =_getTopLineColor(showTop) ;
    linePain.strokeWidth = lineWidth;
    linePain.strokeCap = StrokeCap.square; //画线的头是方形的
    //画圆上面的线
    canvas.drawLine(Offset(centerX, 0), Offset(centerX, lineHeight), linePain);
    //依据下面的线是否显示来设置是否透明
    linePain.color =_getTopLineColor(showBottom)  ;
    // 画圆下面的线
    canvas.drawLine(
        Offset(centerX, lineHeight), Offset(centerX, size.height), linePain);

    // 创建画圆的画笔
    Paint circlePaint = Paint();
    // circlePaint.color = Colors.teal;

    circlePaint.color = isLightCircle
        ? ColorsUtil.hex2Color(lightLineColor!)
        : ColorsUtil.hex2Color(normalLineColor!);

    circlePaint.style = PaintingStyle.fill;
    // 画中间的圆
    canvas.drawCircle(Offset(centerX, lineHeight), centerX, circlePaint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    if (oldDelegate is LeftLinePainter) {
      LeftLinePainter old = oldDelegate;
      if (old.showBottom != showBottom) {
        return true;
      }
      if (old.showTop != showTop) {
        return true;
      }
      // if (old.isLight != isLight) {
      //   return true;
      // }
      return false;
    }
    return true;
  }

  _getTopLineColor(isShow){
    if(isShow){
      if(isLightLine){
        return ColorsUtil.hex2Color(lightLineColor!);
      }
      return ColorsUtil.hex2Color(normalLineColor!);
    }else{
      return Colors.transparent;
    }
  }
}
