
import 'package:flutter/material.dart';

import 'tabbat_indicator.dart';

/// @author:   wzp_vicky
/// @time:     2022年10月27日23:23:50
/// @version:  V1.1
/// @description:tabbar相关组件，tabbar+tabbarview
/// 主要实现的功能：
///    1.tabbar不是在appbar的bottom下
///    2.tabbar的可滚动（不平分）
///    3.indicator的自定义大小和动画
/// */
///
///
/// @author:   wzp_vicky
/// @time:     2023年08月30日18:34:20
/// @version:  V1.2
/// @description:tabbar增加默认的初始化index
///
/// 关于indicator自定义的问题
///我们想要改变指示器的宽度，官方提供了indicatorSize：字段，
///但是这个字段接受一个TabBarIndicatorSize字段，这个字段并不是具体的宽度值，而是一个枚举值，见下只有两种情况，要么跟tab一样宽，要么跟文本一样宽
///
/// enum TabBarIndicatorSize {
///    宽度和tab控件一样宽
///    tab,
///  宽度和文本一样宽
///   label,
///}
///
///，显然这并不能满足一些产品和UI的需求，比如：宽度要设置成比文本小，
///指示器离文本再近一点，指示器能不能做成小圆点等等， 那么这时候我们就不可以靠官方的字段来实现了
///https://www.cnblogs.com/zhouyong0330/p/14406388.html
///https://www.jianshu.com/p/cd958722cbfd

class VTopTabBarModel {
  String title; // 标题
  final Widget? widget; // 对应的widget
  // final Badge? badge; // 设置badge，不用设置title

  VTopTabBarModel({this.title = "", this.widget /*, this.badge*/
  });
}

class VTopTabBar extends StatefulWidget {
  final List<VTopTabBarModel> tabModelArr; // tab对象数组
  final List<String> tabTitle; // tab对象数组
  final Color? bgColor; // TabBar背景颜色，
  final Color? labelColor; // 选中label颜色  和自定义指示器的颜色  这个优先级高于labelStyle的颜色
  final Color? unselectedLabelColor; // 未选中label颜色F
  final double tabHeight; // tab高度 默认35
  final TextStyle? labelStyle; //未选中的字体样式
  final TextStyle? unselectedLabelStyle; //选中的字体样式
  final Decoration? indicator; // 指示器样式  我自己自定义了一个除非使用者再自定义eg:犟犟爱自己弄
  final bool showCenterLine;
  final Function? switchPageCallBack; // 页面切换的回调，返回index

  final double indicatorWidth;
  final double indicatorBottom;
  final int defaultIndex;

  const VTopTabBar(
      {Key? key,
        required this.tabModelArr,
        required this.tabTitle,
        this.bgColor,
        this.labelColor = Colors.blue,
        this.unselectedLabelColor = Colors.black,
        this.tabHeight = 60.0,
        this.labelStyle,
        this.unselectedLabelStyle,
        this.indicator,
        this.showCenterLine = false,
        this.switchPageCallBack,
        this.indicatorBottom = 5,
        this.defaultIndex=0,
        this.indicatorWidth = 25})
      : super(key: key);

  @override
  _VTopTabBarState createState() => _VTopTabBarState();
}

class _VTopTabBarState extends State<VTopTabBar>
    with SingleTickerProviderStateMixin {
  TabController? _tabController;

  @override
  void initState() {
    super.initState();

    _tabController =
        TabController(length: widget.tabModelArr.length, vsync: this,initialIndex:widget.defaultIndex);
    _tabController!.addListener(() {
      if (widget.switchPageCallBack != null) {
        if (_tabController!.index.toDouble() ==
            _tabController!.animation!.value) {
          widget.switchPageCallBack!(_tabController!.index);
        }
      }
    });
  }

  @override
  void dispose() {
    super.dispose();
    _tabController!.dispose();
  }

  @override
  Widget build(BuildContext context) {
    // const TabBar({
    //   Key? key,
    //   required this.tabs,// tab组件列表
    //   this.controller,// tabBar控制器
    //   this.isScrollable = false,// 是否支持滚动
    //   this.padding,// 内部tab内边距
    //   this.indicatorColor,// 指示器颜色
    //   this.automaticIndicatorColorAdjustment = true,// 指示器颜色是否自动跟随主题颜色
    //   this.indicatorWeight = 2.0,// 指示器高度
    //   this.indicatorPadding = EdgeInsets.zero,// 指示器padding
    //   this.indicator,//选择指示器样式
    //   this.indicatorSize,//选择指示器大小
    //   this.labelColor,// 选择标签文本颜色
    //   this.labelStyle,// 选择标签文本样式
    //   this.labelPadding,// 整体标签边距
    //   this.unselectedLabelColor,//未选中标签颜色
    //   this.unselectedLabelStyle,// 未选中标签样式
    //   this.dragStartBehavior = DragStartBehavior.start,//设置点击水波纹效果 跟随全局点击效果
    //   this.overlayColor,// 设置水波纹颜色
    //   this.mouseCursor, // 鼠标指针悬停的效果 App用不到
    //   this.enableFeedback,// 点击是否反馈声音触觉。
    //   this.onTap,// 点击Tab的回调
    //   this.physics,// 滚动边界交互
    // })

    return Column(
      children: [
        Container(
          color: Colors.white,
          width: double.infinity,
          child: TabBar(
            isScrollable: true,

            controller: _tabController,
            tabs: widget.tabTitle
                .map((item) => Tab(
              text: item,
              height: widget.tabHeight,
              // icon: Icon(Icons.add)
            ))
                .toList(),
/*
            tabs: widget.tabModelArr
                .map((item) => Tab(
                      text: item.title,
                      height: widget.tabHeight,
                      // icon: Icon(Icons.add)
                    ))
                .toList(),
*/
            onTap: (int index) {},
            labelColor: widget.labelColor,
            unselectedLabelColor: widget.unselectedLabelColor,
            labelStyle: widget.labelStyle ?? TextStyle(fontSize: 19),
            unselectedLabelStyle: widget.labelStyle ?? TextStyle(fontSize: 10),
            indicator: VTabIndicator(
                tabController: _tabController,
                borderSide: BorderSide(
                    width: 10.0, color: widget.labelColor ?? Colors.black),
                indicatorWidth: widget.indicatorWidth,
                indicatorBottom: widget.indicatorBottom),
          ),
        ),
        Expanded(
            flex: 1,
            child: TabBarView(
              controller: _tabController,
              children: widget.tabModelArr.map((item) => item.widget!).toList(),
            ))
      ],
    );
  }
}

/**
    const TabBar({
    Key key,
    @required this.tabs,// 显示的标签内容，一般使用Tab对象,也可以是其他的Widget
    this.controller,// TabController对象
    this.isScrollable = false,// 是否可滚动
    this.indicatorColor,// 指示器颜色
    this.indicatorWeight = 2.0,// 指示器高度
    this.indicatorPadding = EdgeInsets.zero,// 底部指示器的Padding
    this.indicator,// 指示器decoration，例如边框等
    this.indicatorSize,// 指示器大小计算方式，TabBarIndicatorSize.label跟文字等宽,TabBarIndicatorSize.tab跟每个tab等宽
    this.labelColor,// 选中label颜色
    this.labelStyle,// 选中label的Style
    this.labelPadding,// 每个label的padding值
    this.unselectedLabelColor,// 未选中label颜色
    this.unselectedLabelStyle,// 未选中label的Style
    }) : assert(tabs != null),
    assert(isScrollable != null),
    assert(indicator != null || (indicatorWeight != null && indicatorWeight > 0.0)),
    assert(indicator != null || (indicatorPadding != null)),
    super(key: key);
 */

    