import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:v_base_framework/utils/colors_util.dart';
import 'package:wechat_assets_picker/wechat_assets_picker.dart';
// import 'package:heic_to_jpg/heic_to_jpg.dart';
import './asset_picker_text_delegate.dart';

// import 'package:path/path.dart' as p;
import 'package:wechat_camera_picker/wechat_camera_picker.dart';
/// @author:   wzp_vicky
/// @time:     2023年9月3日10:43:11
/// @version:  V1.1
/// @description：兼容选择图片和拍照的

//使用注意事项

// Android
// 确保 compileSdkVersion 和 targetSdkVersion 升级到 33。 否则，在 Android 13 设备上将有可能无法加载任何资源。
// 权限：READ_EXTERNAL_STORAGE 、WRITE_EXTERNAL_STORAGE 、ACCESS_MEDIA_LOCATION（读取 EXIF 时必需）、READ_MEDIA_IMAGES(读取图片时必需)、READ_MEDIA_VIDEO（读取视频时必需）、READ_MEDIA_AUDIO（读取音频时必需）
//      如果你的目标 SDK 版本大于 33，且你不需要获取图片、视频或者音频， 你可以考虑只声明需要的权限READ_MEDIA_IMAGES、READ_MEDIA_VIDEO

//ios
//1.在 ios/Podfile 中指定最低构建版本至 11.0
//    platform :ios, '11.0'
//2.将以下内容添加至 Info.plist。
// <key>NSAppTransportSecurity</key>
// <dict>
// <key>NSAllowsArbitraryLoads</key>
// <true/>
// </dict>
// <key>NSPhotoLibraryUsageDescription</key>
// <string>你的相册权限描述</string>


class AssetsPickerUtils {
  static toSelectPic(BuildContext context,
      {String language = "zh",
      int maxAssetsCount = 1,
      AssetPickerConfig? otherConfig,
      List<AssetEntity>? selectedAssets,
      int gridCount = 4,
      String? themeColor,
      bool needCamera = true,
      Function? selectCallback}) async {
    // String language = ProSPUtils.getSystemLanguage();
    AssetPickerTextDelegate textDelegate;
    switch (language) {
      case "zh":
        textDelegate = const AssetPickerTextDelegate();
        break;
      case "he":
        textDelegate = const HebrewAssetPickerTextDelegate();
        break;
      case "de":
        textDelegate = const GermanAssetPickerTextDelegate();
        break;
      case "ru":
        textDelegate = const RussianAssetPickerTextDelegate();
        break;
      case "ar":
        textDelegate = const ArabicAssetPickerTextDelegate();
        break;
      case "vi":
        textDelegate = const VietnameseAssetPickerTextDelegate();
        break;
      case "tr":
        textDelegate = const TurkishAssetPickerTextDelegate();
        break;
      case "ja":
        textDelegate = const JapaneseAssetPickerTextDelegate();
        break;
      case "fr":
        textDelegate = const FrenchAssetPickerTextDelegate();
        break;
      default:
        textDelegate = const EnglishAssetPickerTextDelegate();
        break;
    }

    List<AssetEntity>? assets = await AssetPicker.pickAssets(
      context,
      pickerConfig: AssetPickerConfig(
          selectedAssets: selectedAssets,
          maxAssets: maxAssetsCount,
          requestType: RequestType.image,
          gridCount: gridCount,
          themeColor: themeColor == null ? null : ColorsUtil.hex2Color(themeColor!),
          pickerTheme: otherConfig?.pickerTheme,
          textDelegate: textDelegate,
          specialItemPosition: needCamera ? SpecialItemPosition.prepend : SpecialItemPosition.none,
          // specialItemBuilder: needCamera ? _createCamera(context,textDelegate) : null
          specialItemBuilder: !needCamera ?null:(BuildContext context,
              AssetPathEntity? path,
              int length,){
            if (path?.isAll != true) {
              return null;
            }
            return _createCamera(context,textDelegate);
          }

      ),
    );

    if (assets == null || assets.isEmpty) return;
   /*
    卡通项目中的处理heic 图片  暂时拿掉
    AssetEntity _entity = assets[0];
    File file = await _entity.file;
    if (Platform.isIOS) {
      String fileExtension = p.extension(file.path).replaceAll('.', '');
      if (fileExtension.toLowerCase() == 'heic') {
        print('convert to jpeg');
        String jpegPath = await HeicToJpg.convert(file.path);
        file = File(jpegPath);
      }
    }*/
    // AssetEntity _entity = assets[0];
    if (selectCallback != null) {
      // selectCallback(file, _entity);
      // selectCallback(null, _entity);
      selectCallback( assets);
    }
  }

  static _createCamera(
    BuildContext context, AssetPickerTextDelegate textDelegate,{
    bool enableRecording = false, //选择器是否可以录像
    bool onlyEnableRecording = false, //选择器是否仅可以录像。只在 enableRecording 为 true 时有效。
  }) {
    return Semantics(
      label: textDelegate.sActionUseCameraHint,
      button: true,
      onTapHint: textDelegate.sActionUseCameraHint,
      child: GestureDetector(
        behavior: HitTestBehavior.opaque,
        onTap: () async {
          final AssetEntity? result = await CameraPicker.pickFromCamera(
            context,
            pickerConfig: CameraPickerConfig(
              enableRecording: enableRecording,
              onlyEnableRecording: onlyEnableRecording,
            ),
          );
          if (result == null) {
            return;
          }
          final AssetPicker<AssetEntity, AssetPathEntity> picker = context.findAncestorWidgetOfExactType()!;
          final DefaultAssetPickerBuilderDelegate builder = picker.builder as DefaultAssetPickerBuilderDelegate;
          final DefaultAssetPickerProvider p = builder.provider;
          // p.currentAssets=[result];
          // l.add(result);
          // l.addAll(p.currentAssets);


          /*卡通里面是这样洗的
          List<AssetEntity> ll = [result];
          ll.addAll(p.currentAssets);
          p.currentAssets = ll;*/



          // p.currentPath = await p.currentPath.obtainForNewProperties();
          // await p.switchPath( p.currentPathEntity);
          // p.selectAsset(result);

          // Navigator.of(context).pop();


          await p.switchPath(
            PathWrapper<AssetPathEntity>(
              path:
              await p.currentPath!.path.obtainForNewProperties(),
            ),
          );
          p.selectAsset(result);

        },
        child: const Center(
          child: Icon(Icons.camera_enhance, size: 42.0),
        ),
      ),
    );
  }
}
